//---------------------------------------------------------------------------
void __fastcall TInstallForm::InstallBCB1( int lib )
{
TRegistry* registry = new TRegistry();
if (registry==NULL)
    {
    Error( "\
--------------------------------------------\n\
BCB1 Install Components Problem:\n\
--------------------------------------------\n\
\n\
The BCB1 components cannot be installed because there is not enough memory available for use by this application.\n\
Try reducing the number of applications you currently have open, or buy more memory.\n\
\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should install successfully.\n\
\n\
Error Code: BCB1_1\
\n\
--------------------------------------------\n\
" );
    return;
    }
registry->LazyWrite = false;
registry->RootKey = HKEY_CURRENT_USER;

// create a string representing current library components separated with semicolons
AnsiString Library_Components = "";
int first = 0;
for (int i=0;i<lib;i++) first += Library_Num_Components[i];
for (int i=0;i<Library_Num_Components[lib];i++)
    if (Library_Components=="") Library_Components = Library_Component_Names[first+i];
    else Library_Components = Library_Components + ";" + Library_Component_Names[first+i];


// install below

// note:
// BCB1 stores it's component palette in a file called cmplib32.ccl
// (it's much like a large .obj or .lib file)
// This cmplib32.ccl file is stored statically between builds and must be
// recompiled by BCB1.
// There is really nothing much we can install into BCB1, except to tell the
// palette what it should compile next time.

try
    {
    // add what we can to the component palette
    registry->OpenKey( BCB_Keys[BCB1]+"\\cmplib32.ccl.Palette", false );
    if (registry->ValueExists( Library_Abbrevs[lib] )) registry->DeleteValue( Library_Abbrevs[lib] );
//    registry->WriteString( Library_Abbrevs[lib], Library_Components );
    registry->CloseKey();

    // modify global SearchPath set up in BCB1->Options->Environment
    registry->OpenKey( BCB_Keys[BCB1]+"\\Library", false );
    AnsiString old_search_path = registry->ReadString( "SearchPath" );    // NOTE: BCB1 is 'SearchPath' (1 word), BCB3 and BCB4 and BCB5 it's 'Search Path' (2 words)
    AnsiString new_search_path = "";
    TStrings* elem_strings = ElementStrings( old_search_path, ";" );
    if (elem_strings!=NULL)
        {
        for (int i=0;i<elem_strings->Count;i++)
            if (elem_strings->Strings[i].UpperCase().Pos(Library_Names[lib].UpperCase())!=0)
                if (new_search_path=="") new_search_path = elem_strings->Strings[i];
                else new_search_path = new_search_path + ";" + elem_strings->Strings[i];

        if ((new_search_path!=old_search_path) &&
            (new_search_path!="")) registry->WriteString( "SearchPath", new_search_path );
        delete elem_strings;
        }
    registry->CloseKey();
    }
catch (...)
    {
    Error( "\
--------------------------------------------\n\
BCB1 Install Components Problem:\n\
--------------------------------------------\n\
\n\
The BCB1 components cannot be installed because registry entries for BCB1 have been corrupted.\n\
\n\
You may need to reinstall or reconfigure BCB1 to correct this problem.\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" components should install successfully.\n\
\n\
Error Code: BCB1_2\
\n\
--------------------------------------------\n\
" );
    }
delete registry;
}

