//---------------------------------------------------------------------------
void __fastcall TInstallForm::InstallBCB4( int lib )
{
// continue as normal
TRegistry* registry = new TRegistry();
if (registry==NULL)
    {
    Error( "\
--------------------------------------------\n\
BCB4 Install Components Problem:\n\
--------------------------------------------\n\
\n\
The BCB4 components cannot be installed because there is not enough memory available for use by this application.\n\
Try reducing the number of applications you currently have open, or buy more memory.\n\
\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should install successfully.\n\
\n\
Error Code: BCB4_1\
\n\
--------------------------------------------\n\
" );
    return;
    }
registry->LazyWrite = false;
registry->RootKey = HKEY_CURRENT_USER;

TStringList* strings = new TStringList();
if (strings==NULL)
    {
    Error( "\
--------------------------------------------\n\
BCB4 Install Components Problem:\n\
--------------------------------------------\n\
\n\
The BCB4 components cannot be installed because there is not enough memory available for use by this application.\n\
Try reducing the number of applications you currently have open, or buy more memory.\n\
\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should install successfully.\n\
\n\
Error Code: BCB4_2\
\n\
--------------------------------------------\n\
" );
    delete registry;
    return;
    }

// create a string representing currently library components separated with semicolons
AnsiString Library_Components = "";
int first = 0;
for (int i=0;i<lib;i++) first += Library_Num_Components[i];
for (int i=0;i<Library_Num_Components[lib];i++)
    if (Library_Components=="") Library_Components = Library_Component_Names[first+i];
    else Library_Components = Library_Components + ";" + Library_Component_Names[first+i];


// install below

// note:
// BCB4 stores it's component palette in the system registry and can be
// modified easily. Simply insert and modify the appropriate keys.
// Next time BCB4 starts, the new components will be installed
// note: also need to add all TDx_ references to Known + Disabled Packages key

try
    {
    // add to component palette
    // eg. "TDx_Draw" -> "TDx_Draw;TDx_DrawClipper;TDx_DrawColorControl;etc."
    registry->OpenKey( BCB_Keys[BCB4]+"\\Palette", false );
    if (registry->ValueExists( Library_Abbrevs[lib] )) registry->DeleteValue( Library_Abbrevs[lib] );
    registry->WriteString( Library_Abbrevs[lib], Library_Components );
    registry->CloseKey();

    // add to known packages
    // eg. "C:\Program Files\Borland\CBuilder4\Projects\Bpl\TDx_Draw_Library.bpl"
    registry->OpenKey( BCB_Keys[BCB4]+"\\Known Packages", false );
    if (registry->ValueExists( BCB_Root_Dirs[BCB4]+"\\Projects\\Bpl\\"+Library_Names[lib]+".bpl" ))
        registry->DeleteValue( BCB_Root_Dirs[BCB4]+"\\Projects\\Bpl\\"+Library_Names[lib]+".bpl" );
    registry->WriteString( BCB_Root_Dirs[BCB4]+"\\Projects\\Bpl\\"+Library_Names[lib]+".bpl", Library_Names[lib]+" v"+Library_Versions[lib] );
    registry->CloseKey();

    // remove from disabled packages
    registry->OpenKey( BCB_Keys[BCB4]+"\\Disabled Packages", false );
    registry->GetValueNames( strings );
    for (int i=0;i<strings->Count;i++)
        if (strings->Strings[i].UpperCase().Pos(Library_Names[lib].UpperCase())!=0)
            registry->DeleteValue( strings->Strings[i] );
    registry->CloseKey();

    // modify global Search Path set up in BCB4->Environment Options
    registry->OpenKey( BCB_Keys[BCB4]+"\\Library", false );
    AnsiString old_search_path = registry->ReadString( "Search Path" );    // NOTE: BCB1 is 'SearchPath' (1 word), BCB3 and BCB4 and BCB5 it's 'Search Path' (2 words)
    AnsiString new_search_path = "";
    TStrings* elem_strings = ElementStrings( old_search_path, ";" );
    if (elem_strings!=NULL)
        {
        for (int i=0;i<elem_strings->Count;i++)
            if (elem_strings->Strings[i].UpperCase().Pos(Library_Names[lib].UpperCase())!=0)
                if (new_search_path=="") new_search_path = elem_strings->Strings[i];
                else new_search_path = new_search_path + ";" + elem_strings->Strings[i];

        if ((new_search_path!=old_search_path) &&
            (new_search_path!="")) registry->WriteString( "Search Path", new_search_path );
        delete elem_strings;
        }
    registry->CloseKey();
    }
catch (...)
    {
    Error( "\
--------------------------------------------\n\
BCB4 Install Components Problem:\n\
--------------------------------------------\n\
\n\
The BCB4 components cannot be installed because registry entries for BCB4 have been corrupted.\n\
\n\
You may need to reinstall or reconfigure BCB4 to correct this problem.\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" components should install successfully.\n\
\n\
Error Code: BCB4_4\
\n\
---------------------------\n\
" );
    }
delete strings;
delete registry;
}
