//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "TInstallForm.h"
#include "TMainForm.h"
#include "TErrorForm.h"
#include "MiscellaneousFunctions.h"
#include "TUnInstallForm.h"
#include "registry.hpp"
#include "stdio.h"
#include "fcntl.h"
#include "io.h"
#include "dir.h"
#include "sys\stat.h"
#include "clipbrd.hpp"
#include "process.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "TDx_Sound_Library_Install"
#pragma link "TDx_Sound"
#pragma link "TDx_SoundBuffer"
#pragma link "TDSBufferDesc"
#pragma link "PERFGRAP"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
TInstallForm *InstallForm;
//---------------------------------------------------------------------------
#define ACTION( color, str )        {\
                                    Action->Font->Color = (color);\
                                    Action->Caption = (str);\
                                    Action->Update();\
                                    }\

#define RESULT( str )               {\
                                    try {\
                                        Results->Items->Add( (str) );\
                                        }\
                                    catch (Exception &e)\
                                        {\
                                        try {\
                                            Results->Items->Clear();\
                                            Results->Items->Add( (str) );\
                                            }\
                                        catch (Exception &e)\
                                            {\
                                            }\
                                        }\
                                    if (Results->Items->Count>0) Results->ItemIndex = Results->Items->Count-1;\
                                    Results->Update();\
                                    }\

//---------------------------------------------------------------------------
struct TPackItHeader
    {
    word        Version;
    dword       TotalFiles;
    TDateTime   DateCreated;
    };
//---------------------------------------------------------------------------
struct TPackItFileHeader
    {
    dword       FilePos;
    char        FileName[1024];
    dword       FileSize;
    };
//---------------------------------------------------------------------------
void __fastcall TInstallForm::ExtractFromPackIt( int lib )
{
AnsiString filename = Install_Directory+"\\"+Library_Names[lib].UpperCase()+".PIT";

// --------------------
// extract files...
// --------------------
TPackItHeader header;
TPackItFileHeader fileheader;

int file = open(filename.c_str(),O_RDWR|O_BINARY);
if (file == -1)
    {
    Error( "\
--------------------------------\n\
.PIT File Extraction Problem:\n\
--------------------------------\n\
\n\
This install program normally extracts all required files from the accompanying .PIT file :\n\
\n\
\n  \""+filename+"\"\n\
\n\
This file cannot be read by the install application.\n\
It is possible it either does not exist or is marked with no read access.\n\
\n\
Try replacing the file with a different copy of the "+Library_Names[lib].UpperCase()+".PIT"+" file.\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should install successfully.\n\
\n\
Error Code: PIT_1\n\
\n\
---------------------------\n\
" );
    return;
    }

// read PackIt file header (to get total # of files)
int error = read( file, &header, sizeof(TPackItHeader) );
if (error == -1)
    {
    close(file);
    Error( "\
--------------------------------\n\
.PIT File Extraction Problem:\n\
--------------------------------\n\
\n\
This install program normally extracts all required files from the accompanying .PIT file :\n\
\n\
\n  \""+filename+"\"\n\
\n\
This file exists at the expected location but appears to be corrupted and as such will no longer function correctly.\n\
\n\
Try replacing the file with a different copy of the "+Library_Names[lib].UpperCase()+".PIT"+" file.\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should install successfully.\n\
\n\
Error Code: PIT_2\n\
\n\
---------------------------\n\
" );
    return;
    }

// save the current directory so we can change back later
AnsiString original_dir = GetCurrentDir();

// extract all PackIt files...
FileProgress->Min = 0;
FileProgress->Max = header.TotalFiles;
FileProgress->Position = 0;
FileProgress->Update();

dword num_extracted = 0;
for (dword i=0;i<header.TotalFiles;i++)
    {
    Application->HandleMessage();
    if (!installing) break;

    // read file info
    read( file, &fileheader, sizeof(TPackItFileHeader) );

    // all files come in as :-
    //  eg.
    //      D:\TDx_Library\*.*
    // need to be
    //      O:\TDx_Root_Dir\TDx_Library\*.*

    AnsiString old_filename = fileheader.FileName;
    AnsiString new_filename = ReplaceText( old_filename, "D:\\TDx_Library", TDx_Root_Dir );
    AnsiString just_drive = ElementString( new_filename, ":", 0);
    AnsiString just_filename = ExtractFileName( new_filename );
    AnsiString just_directories = ExtractFileDir( new_filename );

    File->Caption = "[ "+just_filename+" ]";
    File->Update();
    FilePath->Caption = new_filename;
    FilePath->Update();
    FileCount->Caption = IntToStr(i+1) + " of " + IntToStr(header.TotalFiles);
    FileCount->Update();
    FileProgress->Position = i;
    FileProgress->Update();

    // change to relevant drive's root directory
    SetCurrentDir( just_drive+":\\" );

    // split up directories
    TStrings* dirs = ElementStrings( just_directories, "\\" );
    if (dirs==NULL)
        {
        close(file);
        Error( "\
--------------------------------\n\
.PIT File Extraction Problem:\n\
--------------------------------\n\
\n\
This install program normally creates directories required by files in the accompanying .PIT file.\n\
Unfortunately there is not enough available memory to create the directories required by these files.\n\
\n\
Try reducing the number of applications you currently have open, or buy more memory.\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should install successfully.\n\
\n\
Error Code: PIT_3\n\
\n\
---------------------------\n\
" );
        break;
        }

    // traverse the directory and create each subdir as required
    // ignore the first index, it's a null string as such (ie "C:" or "D:")
    for (int i=1;i<dirs->Count;i++)
        {
//            ShowMessage("Now Dir: "+GetCurrentDir()+"\ncreating sub: "+dirs->Strings[i]);
        // if the directory doesn't exist, create it
        if (!DirectoryExists( dirs->Strings[i] ))
            {
            CreateDir( dirs->Strings[i] );
            if (!DirectoryExists( dirs->Strings[i] ))
                {
                delete dirs;
                close(file);
                Error( "\
--------------------------------\n\
.PIT File Extraction Problem:\n\
--------------------------------\n\
\n\
This install program normally creates directories required by files in the accompanying .PIT file.\n\
Unfortunately one of the directories required by a file could not be created successfully :\n\
\n\
  File: \""+filename+"\"\n\
  Problem Directory: \""+dirs->Strings[i]+"\"\n\
\n\
Maybe try removing the directories, or reduce the depth of the relevant directory tree.\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should install successfully.\n\
\n\
Error Code: PIT_4\n\
\n\
---------------------------\n\
" );
                break;
                }
            }
        // directory exists now

        // change to the newly created directory, ready to create the
        // next level down if required
        SetCurrentDir( dirs->Strings[i] );
        }

    // release directory strings
    delete dirs;

    if (FileExists(just_filename))
        {
        RESULT( "> Replacing "+new_filename );

        // if for some reason, it is read-only.... change it to read-write
        chmod( just_filename.c_str(), S_IREAD | S_IWRITE );
        DeleteFile(just_filename);
        if (FileExists(just_filename))
            {
            close(file);
            Error( "\
--------------------------------\n\
.PIT File Extraction Problem:\n\
--------------------------------\n\
\n\
This install program normally deletes existing files also stored in the accompanying .PIT file.\n\
Unfortunately one of the files could not be deleted successfully :\n\
\n\
  File: \""+filename+"\"\n\
\n\
It may be that the file exists but cannot be deleted, try removing the old file manually.\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should install successfully.\n\
\n\
Error Code: PIT_5\n\
\n\
---------------------------\n\
" );
            // ignore this file, handle the next file
            break;
            }
        // continue below normally... // try creating new file
        }
    else
        {
        RESULT( "> Saving "+new_filename );
        }

    // try creating new file
    int tempfile = open( just_filename.c_str(), O_CREAT | O_TRUNC | O_WRONLY | O_BINARY );
    if (tempfile==-1)
        {
        close(file);
        close(tempfile);
        Error( "\
--------------------------------\n\
.PIT File Extraction Problem:\n\
--------------------------------\n\
\n\
This install program normally creates files stored in the accompanying .PIT file.\n\
Unfortunately one of the files could not be created successfully :\n\
\n\
  File: \""+new_filename+"\"\n\
\n\
It may be that the file already exists but could not be deleted, try removing the old file manually.\n\
It may also be that you have no available hard drive space, try making more space available.\n\
\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should install successfully.\n\
\n\
Error Code: PIT_6\n\
\n\
---------------------------\n\
" );
        break;
        }

    // try allocating memory for the relevant file
    long templength = fileheader.FileSize;
    char* tempbuffer = new char[templength];
    if (!tempbuffer)
        {
        close(file);
        close(tempfile);
        delete[] tempbuffer;
        Error( "\
--------------------------------\n\
.PIT File Extraction Problem:\n\
--------------------------------\n\
\n\
This install program normally creates files stored in the accompanying .PIT file.\n\
Unfortunately one of the files could not be created successfully :\n\
\n\
  File: \""+new_filename+"\"\n\
\n\
There is not enough memory available to duplicate the file.\n\
Try reducing the number of applications you currently have open, or buy more memory.\n\
\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should install successfully.\n\
\n\
Error Code: PIT_7\n\
\n\
---------------------------\n\
" );
        break;
        }

    // read entire file
    read( file, tempbuffer, fileheader.FileSize );

    // save it to disk....
    write( tempfile, tempbuffer, templength );
    close( tempfile );

    // for some reason, it can be created as read-only....
    // make sure it is read-write
    chmod( just_filename.c_str(), S_IREAD | S_IWRITE );

    num_extracted++;
    delete[] tempbuffer;

    }
close(file);

// restore to original drive/directory
SetCurrentDir( original_dir );

if (num_extracted != header.TotalFiles)
    {
    Error( "\
--------------------------------\n\
.PIT File Extraction Problem:\n\
--------------------------------\n\
\n\
This install program normally creates all files stored in the accompanying .PIT file.\n\
Unfortunately some of the files could not be created successfully :\n\
\n\
  Number of Files extracted : "+IntToStr(num_extracted)+"\n\
  Total Number of Files meant to be extracted : "+IntToStr(header.TotalFiles)+"\n\
\n\
IMPORTANT NOTE:\n\
\n\
  Until all install problems are resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" will not function successfully.\n\
\n\
Error Code: PIT_8\n\
\n\
---------------------------\n\
" );
    }
}
//---------------------------------------------------------------------------
__fastcall TInstallForm::TInstallForm(TComponent* Owner)
        : TForm(Owner)
{
installing = false;
InstallComplete = false;
}
//---------------------------------------------------------------------------
void __fastcall TInstallForm::ExitClick(TObject *Sender)
{
if (MainForm->Goodbye->Play(0,0))
while (MainForm->Goodbye->Playing) ;  // wait for end playing
Application->Terminate();
}
//---------------------------------------------------------------------------

void __fastcall TInstallForm::InstallClick(TObject *Sender)
{
if (MainForm->Next->Playing) MainForm->Next->Stop();
MainForm->Next->Play(0,0);

if (installing)
    {
    installing = false;

    if (Action->Caption!="> Install Complete") ACTION( clMaroon, "> Install Aborted" );
    RESULT( "------------------------------------------------------------------------------------------------" );
    RESULT( Action->Caption );
    RESULT( "------------------------------------------------------------------------------------------------" );

    try {
        AnsiString name = Install_Directory + "\\TDx_Install.log";
        if (FileExists( name )) DeleteFile( name );
        Results->Items->SaveToFile( name );
        RESULT( "> Saved install results to "+name );
        }
   catch (...)
        {
        }

    MainForm->InstallWav->Stop();

    Install->Caption = "> start";
    Install->Hint = "Begin the installation.";
    Install->Update();
    Installing->Caption = "-";
    Installing->Hint = "Ready to install";
    Installing->Update();

    Actions->Visible = true;
    Install->Visible = true;
    InstallImage->Visible = true;
    Prev->Visible = true;
    PrevImage->Visible = true;
    Exit->Visible = true;
    ExitImage->Visible = true;
    ExitHere->Visible = true;
    }
else
    {
    InstallComplete = false;
    MainForm->InstallWav->Play(0,DSBPLAY_LOOPING);

    TRegistry* registry = new TRegistry();
    if (registry==NULL)
        {
        Error( "\
------------------\n\
Install Problem:\n\
------------------\n\
\n\
The components cannot be installed because there is not enough memory available to access the system registry.\n\
Try reducing the number of applications you currently have open, or buy more memory.\n\
\n\
Once the problem is resolved the TDx_Library should install successfully.\n\
\n\
Error Code: INSTALL_1\n\
\n\
---------------------------\n\
" );
        return;
        }
    registry->RootKey = HKEY_LOCAL_MACHINE;
    registry->LazyWrite = false;

    // create default keys if they don't yet exist
    if (!TDx_Installed)
        {
        registry->OpenKey( TDx_Root_Key, true );
        registry->WriteString( "Root_Dir", TDx_Root_Dir );
        registry->WriteDate( "First_Installed", TDx_First_Installed );
        registry->CloseKey();
        }

    installing = true;
    Results->Items->Clear();
    CopiedLabel->Caption = "( Copy text to Clipboard )";
    CopiedLabel->Visible = false;
    SpeedButton1->Visible = false;
    Application->HandleMessage();

    ACTION( clGreen, "> Install Started" );
    RESULT( "------------------------------------------------------------------------------------------------" );
    RESULT( Action->Caption );

    Install->Caption = "[] STOP";
    Install->Hint = "Stop the installation [NOT RECOMMENDED] ";
    Install->Update();
    Installing->Caption = "-";
    Installing->Hint = "Currently installing";
    Installing->Update();

    Prev->Visible = false;
    PrevImage->Visible = false;
    Exit->Visible = false;
    ExitImage->Visible = false;
    ExitHere->Visible = false;

    // Copy DirectX sdk files to all selected bcb versions
    if (true)
        {
        bool copied_libs[NUM_BCB];
        bool copied_includes[NUM_BCB];
        for (int bcb=0;bcb<NUM_BCB;bcb++)
            {
            copied_libs[bcb] = false;
            copied_includes[bcb] = false;
            }

        BCB->Caption = "-";
        BCB->Update();
        BCBCount->Caption = "-";
        BCBCount->Update();
        BCBProgress->Position = 0;
        BCBProgress->Update();

        for (int lib=0;lib<NUM_LIBS;lib++)
            for (int bcb=0;bcb<NUM_BCB;bcb++)
                {
                Application->HandleMessage();
                if (!installing) break;

                BCB->Caption = "[ "+BCB_Names[bcb]+" ]";
                BCB->Update();
                BCBCount->Caption = IntToStr(bcb+1) + " of " + IntToStr(NUM_BCB);
                BCBCount->Update();
                BCBProgress->Position = bcb+1;
                BCBProgress->Update();

                // Copy DirectX Include files
                if (!installing) break;
                if (!copied_includes[bcb] && BCB_Exists[bcb])
                    {
                    RESULT( "------------------------------------------------------------------------------------------------" );
                    ACTION( clGreen, "> Copying DirectX 7.0a Include files to "+BCB_Names[bcb]+"\\Include" );
                    RESULT( Action->Caption );
                    RESULT( "------------------------------------------------------------------------------------------------" );

                    copied_includes[bcb] = true;
                    CopyDirectXIncludeFiles( lib, bcb );
                    }

                // Copy DirectX Lib files
                if (!installing) break;
                if (!copied_libs[bcb] && BCB_Exists[bcb])
                    {
                    RESULT( "------------------------------------------------------------------------------------------------" );
                    ACTION( clGreen, "> Copying DirectX 7.0a Lib files to "+BCB_Names[bcb]+"\\Lib" );
                    RESULT( Action->Caption );
                    RESULT( "------------------------------------------------------------------------------------------------" );

                    copied_libs[bcb] = true;
                    CopyDirectXLibFiles( lib, bcb );
                    }
                }
        }

    // install procedure
    for (int lib=0;lib<NUM_LIBS;lib++)
        {
        Application->HandleMessage();
        if (!installing) break;

        bool install_now = false;
        switch (lib)
            {
            case DRAW:   install_now = true; break;
            case SOUND:  install_now = true; break;
            case INPUT:  install_now = true; break;
            case PLAY:   install_now = true; break;
            case D3I:    install_now = true; break;
            }

        if (install_now)
            {
            RESULT( "------------------------------------------------------------------------------------------------" );
            ACTION( clGreen, "> Installing "+Library_Names[lib] );
            RESULT( Action->Caption );
            RESULT( "------------------------------------------------------------------------------------------------" );

            Installing->Caption = Library_Names[lib];
            Installing->Hint = "Installing "+Library_Names[lib];
            Installing->Update();
            }

        Lib->Caption = "[ "+Library_Names[lib]+" ]";
        Lib->Update();
        LibCount->Caption = IntToStr(lib+1) + " of " + IntToStr(NUM_LIBS);
        LibCount->Update();
        LibProgress->Position = lib+1;
        LibProgress->Update();

        BCB->Caption = "-";
        BCB->Update();
        BCBCount->Caption = "-";
        BCBCount->Update();
        BCBProgress->Position = 0;
        BCBProgress->Update();

        File->Caption = "-";
        File->Update();
        FilePath->Caption = "-";
        FilePath->Update();
        FileCount->Caption = "-";
        FileCount->Update();
        FileProgress->Position = 0;
        FileProgress->Update();

        // Extract Files
        if (install_now)
            {
            RESULT( "------------------------------------------------------------------------------------------------" );
            ACTION( clGreen, "> Extracting "+Library_Names[lib]+" Files:" );
            RESULT( Action->Caption );
            RESULT( "------------------------------------------------------------------------------------------------" );

            ExtractFromPackIt( lib );
            if (ErrorOccurred)
                {
                InstallClick(this);
                delete registry;
                return;
                }
            }

        // install for all BCB's
        for (int bcb=0;bcb<NUM_BCB;bcb++)
            {
            Application->HandleMessage();
            if (!installing) break;

            BCB->Caption = "[ "+BCB_Names[bcb]+" ]";
            BCB->Update();
            BCBCount->Caption = IntToStr(bcb+1) + " of " + IntToStr(NUM_BCB);
            BCBCount->Update();
            BCBProgress->Position = bcb+1;
            BCBProgress->Update();

            // Copy TDx Include Files
            if (install_now && BCB_Exists[bcb])
                {
                RESULT( "------------------------------------------------------------------------------------------------" );
                ACTION( clGreen, "> Copying "+Library_Names[lib]+" Include Files to "+BCB_Names[bcb] );
                RESULT( Action->Caption );
                RESULT( "------------------------------------------------------------------------------------------------" );

                CopyIncludeFiles( lib, bcb );
                }

            // Copy TDx Lib Files
            if (!installing) break;
            if (install_now && BCB_Exists[bcb])
                {
                RESULT( "------------------------------------------------------------------------------------------------" );
                ACTION( clGreen, "> Copying "+Library_Names[lib]+" Library Files to "+BCB_Names[bcb] );
                RESULT( Action->Caption );
                RESULT( "------------------------------------------------------------------------------------------------" );

                CopyLibFiles( lib, bcb );
                }

            // Copy TDx Project Files
            if (!installing) break;
            if (install_now && BCB_Exists[bcb])
                {
                RESULT( "------------------------------------------------------------------------------------------------" );
                ACTION( clGreen, "> Copying "+Library_Names[lib]+" Project Files to "+BCB_Names[bcb] );
                RESULT( Action->Caption );
                RESULT( "------------------------------------------------------------------------------------------------" );

                CopyProjectFiles( lib, bcb );
                }

            // Copy TDx Source Code
            if (!installing) break;
            if (install_now && BCB_Exists[bcb])
                {
                RESULT( "------------------------------------------------------------------------------------------------" );
                ACTION( clGreen, "> Copying "+Library_Names[lib]+" Source Code to "+BCB_Names[bcb] );
                RESULT( Action->Caption );
                RESULT( "------------------------------------------------------------------------------------------------" );

                CopySourceCodeFiles( lib, bcb );
                }

            // UNInstall Components
            if (!installing) break;
            if (install_now && BCB_Exists[bcb] && Library_Installed[lib][bcb] && (UnInstallForm!=NULL))
                {
                RESULT( "------------------------------------------------------------------------------------------------" );
                ACTION( clGreen, "> Removing existing "+Library_Names[lib]+" Components from "+BCB_Names[bcb] );
                RESULT( Action->Caption );
                RESULT( "------------------------------------------------------------------------------------------------" );

                UnInstallForm->UnRegisterComponents( lib, bcb );
                }

            // Install Components
            if (!installing) break;
            if (install_now && BCB_Exists[bcb])
                {
                RESULT( "------------------------------------------------------------------------------------------------" );
                ACTION( clGreen, "> Installing "+Library_Names[lib]+" Components into "+BCB_Names[bcb]+":" );
                RESULT( Action->Caption );
                RESULT( "------------------------------------------------------------------------------------------------" );

                if (lib!=DRAW) RegisterComponents( lib, bcb );
                else
                    {
                    if (Library_Package_Exists[D3I])
                        {
                        RESULT( "------------------------------------------------------------------------------------------------" );
                        ACTION( clGreen, "> Skipping "+Library_Names[lib]+" Components into "+BCB_Names[bcb]+":" );
                        RESULT( Action->Caption );
                        RESULT( "------------------------------------------------------------------------------------------------" );
                        }
                    else
                        {
                        RegisterComponents( lib, bcb );
                        }
                    }
                }

            // UNInstall Component Help Files
            if (!installing) break;
            if (install_now && BCB_Exists[bcb] && Library_Installed_Help[lib][bcb] && (UnInstallForm!=NULL))
                {
                RESULT( "------------------------------------------------------------------------------------------------" );
                ACTION( clGreen, "> Removing existing "+Library_Names[lib]+" Component Help from "+BCB_Names[bcb] );
                RESULT( Action->Caption );
                RESULT( "------------------------------------------------------------------------------------------------" );

                UnInstallForm->UnRegisterComponentHelp( lib, bcb );
                }

            // Install Component Help Files
            if (!installing) break;
            if (install_now && BCB_Exists[bcb])
                {
                RESULT( "------------------------------------------------------------------------------------------------" );
                ACTION( clGreen, "> Installing "+Library_Names[lib]+" Help Files into "+BCB_Names[bcb]+":" );
                RESULT( Action->Caption );
                RESULT( "------------------------------------------------------------------------------------------------" );

                RegisterComponentHelp( lib, bcb );
                }

            // Install onto Start Menu
//            if (!installing) break;
//            if (install_now && BCB_Exists[bcb] && Library_CheckBoxes[lib][bcb]->Checked && InstallSetupForm->StartMenu->Checked)
//                {
//                RESULT( "------------------------------------------------------------------------------------------------" );
//                ACTION( clGreen, "> Installing "+Library_Names[lib]+" for "+BCB_Names[bcb]+" into the Start Menu." );
//                RESULT( Action->Caption );
//                RESULT( "------------------------------------------------------------------------------------------------" );
//
//                RegisterStartMenu( lib, bcb );
//                }
            }
        }

    // finished
    delete registry;

    Lib->Caption = "-";
    Lib->Update();
    LibCount->Caption = "-";
    LibCount->Update();
    LibProgress->Position = 0;
    LibProgress->Update();

    BCB->Caption = "-";
    BCB->Update();
    BCBCount->Caption = "-";
    BCBCount->Update();
    BCBProgress->Position = 0;
    BCBProgress->Update();

    File->Caption = "-";
    File->Update();
    FilePath->Caption = "-";
    FilePath->Update();
    FileCount->Caption = "-";
    FileCount->Update();
    FileProgress->Position = 0;
    FileProgress->Update();

    if (installing) // user has not clicked abort
        {
        bool any_installed_into_bcb1 = false;
        AnsiString libs = "";
        for (int lib=0;lib<NUM_LIBS;lib++)
            {
            any_installed_into_bcb1 |= Library_Installed[lib][BCB1];
            if (Library_Installed[lib][BCB1])
                libs = libs + "          Add->Browse-> \n          \""+BCB_Root_Dirs[BCB1]+"\\Lib\\"+Library_Names[lib]+"_Install.CPP\"->\n";
            }
        if (any_installed_into_bcb1)
            {
            MainForm->InstallWav->Stop();
            MainForm->BCBCheck = false;
            Error("\
This application cannot fully automate the install for BCB1.\n\
For a successful install, complete the following steps:-\n\
\n\
  (1) Open BCB1\n\
\n\
  (2) Close All Files\n\
      [ File->Close All ]\n\
\n\
  (3) Install the library\n\
\n\
      * if all selected BCB1 libraries are currently installed\n\
        [ Component->Rebuild Library]\n\
\n\
      * otherwise, use the commands below to install the previously selected libraries\n\
        [ Component->Install->\n\
"+libs+"\
          Ok ]\n\
\n\
  (4) Close BCB1\n\
\n\
When you have finished the steps shown above, click OK.");
            ErrorOccurred = false;
            MainForm->BCBCheck = true;
            MainForm->InstallWav->Play(0,DSBPLAY_LOOPING);
            }

        RESULT( "------------------------------------------------------------------------------------------------" );
        ACTION( (TColor) 0x00D03000, "> Install Complete" );
        RESULT( "------------------------------------------------------------------------------------------------" );
        MainForm->Completed->Play(0,0);

        Installing->Caption = "-";
        Installing->Hint = "Ready to install";
        Installing->Update();

        SpeedButton1->Visible = true;
        CopiedLabel->Visible = true;

        // just click the install button again, with a little fix there
        InstallComplete = true;
        InstallClick(this);
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TInstallForm::CopyDirectXIncludeFiles( int lib, int bcb )
{
FileListBox1->Mask = "*.*";
try {
    FileListBox1->Directory = DirectX_Root_Dir + "\\Include";
    Installing->Caption = "DirectX Include Files";
    Installing->Hint = "Currently Installing DirectX 7.0a SDK Files";
    Installing->Update();
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find DirectX directory "+DirectX_Root_Dir+"\\Include" );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!installing) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];
    AnsiString dest = BCB_Root_Dirs[bcb] + "\\Include\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = dest;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (!DirectoryExists(BCB_Root_Dirs[bcb]+"\\Include\\TDX_SAVE"))
        {
        RESULT( "> Creating Directory "+BCB_Root_Dirs[bcb]+"\\Include\\TDX_SAVE" );
        CreateBigDir( BCB_Root_Dirs[bcb]+"\\Include\\TDX_SAVE" );
        }
    AnsiString backup_file = BCB_Root_Dirs[bcb] + "\\Include\\TDX_SAVE\\"+FileListBox1->Items->Strings[file];
    if (!FileExists( backup_file ))
        {
        RESULT( "> Backup "+dest );
        FileCopy( dest, backup_file, true );
        }

    if (FileExists( dest ))
        {
        RESULT( "> Replacing "+dest );
        DeleteFile( dest );
        }
    else
        RESULT( "> Saving "+dest );
    FileCopy( srce, dest, true );
    }
}
//---------------------------------------------------------------------------
void __fastcall TInstallForm::CopyDirectXLibFiles( int lib, int bcb )
{
FileListBox1->Mask = "*.*";
try {
    FileListBox1->Directory = DirectX_Root_Dir + "\\Lib\\Borland";
    Installing->Caption = "DirectX Lib Files";
    Installing->Hint = "Currently Installing DirectX 7.0a SDK Lib Files";
    Installing->Update();
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find DirectX directory "+DirectX_Root_Dir+"\\Lib\\Borland" );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!installing) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];
    AnsiString dest = BCB_Root_Dirs[bcb] + "\\Lib\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = dest;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (!DirectoryExists(BCB_Root_Dirs[bcb]+"\\Lib\\TDX_SAVE"))
        {
        RESULT( "> Creating Directory "+BCB_Root_Dirs[bcb]+"\\Lib\\TDX_SAVE" );
        CreateBigDir( BCB_Root_Dirs[bcb]+"\\Lib\\TDX_SAVE" );
        }
    AnsiString backup_file = BCB_Root_Dirs[bcb] + "\\Lib\\TDX_SAVE\\"+FileListBox1->Items->Strings[file];
    if (!FileExists( backup_file ))
        {
        RESULT( "> Backup "+dest );
        FileCopy( dest, backup_file, true );
        }
    if (FileExists( dest ))
        {
        RESULT( "> Replacing "+dest );
        DeleteFile( dest );
        }
    else
        RESULT( "> Saving "+dest );
    FileCopy( srce, dest, true );
    }
}
//---------------------------------------------------------------------------
void __fastcall TInstallForm::CopyIncludeFiles( int lib, int bcb )
{
// copy files from C:\TDx_Library\TDx_Draw_Library\Source\*.H
// to
// C:\Program Files\Borland\Include\*.*

switch (lib)
    {
    case DRAW:      FileListBox1->Mask = "TDD*.H;TDx_Draw*.H"; break;
    case SOUND:     FileListBox1->Mask = "TDS*.H;TDx_Sound*.H"; break;
    case INPUT:     FileListBox1->Mask = "TDI*.H;TDx_Input*.H"; break;
    case PLAY:      FileListBox1->Mask = "TDP*.H;TDx_Play*.H"; break;
    case D3I:       FileListBox1->Mask = "TD3D*.H;TDx_3D*.H"; break;
    }
try {
    FileListBox1->Directory = TDx_Root_Dir + "\\" + Library_Names[lib] + "\\Source";
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find "+Library_Names[lib]+" v"+Library_Versions[lib]+" directory "+TDx_Root_Dir + "\\" + Library_Names[lib] + "\\Source" );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!installing) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];
    AnsiString dest = BCB_Root_Dirs[bcb] + "\\Include\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = dest;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (FileExists( dest ))
        {
        RESULT( "> Replacing "+dest );
        DeleteFile( dest );
        }
    else
        RESULT( "> Saving "+dest );
    FileCopy( srce, dest, true );
    }

// copy files from C:\TDx_Library\TDx_Draw_Library\Source\TDx_Library*.H

switch (lib)
    {
    case DRAW:      FileListBox1->Mask = "TDx_Library*.H"; break;
    case SOUND:     FileListBox1->Mask = "TDx_Library*.H"; break;
    case INPUT:     FileListBox1->Mask = "TDx_Library*.H"; break;
    case PLAY:      FileListBox1->Mask = "TDx_Library*.H"; break;
    case D3I:       FileListBox1->Mask = "TDx_Library*.H"; break;
    }
try {
    FileListBox1->Directory = TDx_Root_Dir + "\\Source";
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find "+Library_Names[lib]+" v"+Library_Versions[lib]+" directory "+TDx_Root_Dir + "\\Source" );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!installing) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];
    AnsiString dest = BCB_Root_Dirs[bcb] + "\\Include\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = dest;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (FileExists( dest ))
        {
        RESULT( "> Replacing "+dest );
        DeleteFile( dest );
        }
    else
        RESULT( "> Saving "+dest );
    FileCopy( srce, dest, true );
    }
}
//---------------------------------------------------------------------------
void __fastcall TInstallForm::CopySourceCodeFiles( int lib, int bcb )
{
// copy files from C:\TDx_Library\TDx_Draw_Library\Source\*.CPP
// to
// C:\Program Files\Borland\Include\*.*

switch (lib)
    {
    case DRAW:      FileListBox1->Mask = "TDD*.CPP;TDx_Draw*.CPP"; break;
    case SOUND:     FileListBox1->Mask = "TDS*.CPP;TDx_Sound*.CPP"; break;
    case INPUT:     FileListBox1->Mask = "TDI*.CPP;TDx_Input*.CPP"; break;
    case PLAY:      FileListBox1->Mask = "TDP*.CPP;TDx_Play*.CPP"; break;
    case D3I:       FileListBox1->Mask = "TD3D*.CPP;TDx_3D*.CPP"; break;
    }
try {
    FileListBox1->Directory = TDx_Root_Dir + "\\" + Library_Names[lib] + "\\Source";
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find "+Library_Names[lib]+" v"+Library_Versions[lib]+" directory "+TDx_Root_Dir + "\\" + Library_Names[lib] + "\\Source" );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!installing) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];
    AnsiString dest = BCB_Root_Dirs[bcb] + "\\Include\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = dest;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (FileExists( dest ))
        {
        RESULT( "> Replacing "+dest );
        DeleteFile( dest );
        }
    else
        RESULT( "> Saving "+dest );
    FileCopy( srce, dest, true );
    }

// copy files from C:\TDx_Library\TDx_Draw_Library\Source\TDx_Library*.CPP

switch (lib)
    {
    case DRAW:      FileListBox1->Mask = "TDx_Library*.CPP;"; break;
    case SOUND:     FileListBox1->Mask = "TDx_Library*.CPP;"; break;
    case INPUT:     FileListBox1->Mask = "TDx_Library*.CPP;"; break;
    case PLAY:      FileListBox1->Mask = "TDx_Library*.CPP;"; break;
    case D3I:       FileListBox1->Mask = "TDx_Library*.CPP;"; break;
    }
try {
    FileListBox1->Directory = TDx_Root_Dir + "\\Source";
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find "+Library_Names[lib]+" v"+Library_Versions[lib]+" directory "+TDx_Root_Dir + "\\Source" );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!installing) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];
    AnsiString dest = BCB_Root_Dirs[bcb] + "\\Include\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = dest;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (FileExists( dest ))
        {
        RESULT( "> Replacing "+dest );
        DeleteFile( dest );
        }
    else
        RESULT( "> Saving "+dest );
    FileCopy( srce, dest, true );
    }
}
//---------------------------------------------------------------------------
void __fastcall TInstallForm::CopyLibFiles( int lib, int bcb )
{
// copy files from C:\TDx_Library\Pre-Compiled\BCB1\*.OBJ
// copy files from C:\TDx_Library\Pre-Compiled\BCB1\*.LIB
// copy files from C:\TDx_Library\Pre-Compiled\BCB1\*.RES
// to
// C:\Program Files\Borland\Lib\*.*

switch (lib)
    {
    case DRAW:      FileListBox1->Mask = "TDD*.OBJ;TDx_Draw*.LIB;TDx_Draw*.OBJ;TDx_Draw*.RES;TDx_Draw*.CPP;TDx_Library*.OBJ;"; break;
    case SOUND:     FileListBox1->Mask = "TDS*.OBJ;TDx_Sound*.LIB;TDx_Sound*.OBJ;TDx_Sound*.RES;TDx_Sound*.CPP;TDx_Library*.OBJ;"; break;
    case INPUT:     FileListBox1->Mask = "TDI*.OBJ;TDx_Input*.LIB;TDx_Input*.OBJ;TDx_Input*.RES;TDx_Input*.CPP;TDx_Library*.OBJ;"; break;
    case PLAY:      FileListBox1->Mask = "TDP*.OBJ;TDx_Play*.LIB;TDx_Play*.OBJ;TDx_Play*.RES;TDx_Play*.CPP;TDx_Library*.OBJ;"; break;
    case D3I:       FileListBox1->Mask = "TD3D*.OBJ;TDx_3D*.LIB;TDx_3D*.OBJ;TDx_3D*.RES;TDx_3D*.CPP;TDx_Library*.OBJ;"; break;
    }
try {
    FileListBox1->Directory = TDx_Root_Dir + "\\Pre-Compiled\\" + BCB_Names[bcb];
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find "+Library_Names[lib]+" v"+Library_Versions[lib]+" directory "+TDx_Root_Dir+"\\Pre-Compiled\\"+BCB_Names[bcb] );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!installing) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];
    AnsiString dest = BCB_Root_Dirs[bcb] + "\\Lib\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = dest;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (FileExists( dest ))
        {
        RESULT( "> Replacing "+dest );
        DeleteFile( dest );
        }
    else
        RESULT( "> Saving "+dest );
    FileCopy( srce, dest, true );
    }
}
//---------------------------------------------------------------------------
void __fastcall TInstallForm::CopyProjectFiles( int lib, int bcb )
{
// project files:
//  all     library.bpl.bpi.bpk.lib
//          library_install.cpp.h
//
//  BCB1\lib\library_install.cpp
//  BCB1\lib\library_install.h
//  BCB1\lib\library_install.obj
//
//  BCB3\lib\library_install.cpp
//  BCB3\lib\library_install.h
//  BCB3\lib\library_install.obj
//  BCB3\lib\library.bpl
//  BCB3\lib\library.bpi
//  BCB3\lib\library.lib
//
//  BCB4\lib\library_install.cpp
//  BCB4\lib\library_install.h
//  BCB4\lib\library_install.obj
//  BCB4\projects\bpl\library.bpl
//  BCB4\projects\lib\library.bpi
//  BCB4\projects\lib\library.lib
//
//  BCB5\lib\library_install.cpp            // start first section
//  BCB5\lib\library_install.h              // end first section
//  BCB5\lib\library_install.obj            // start second section
//  BCB5\projects\bpl\library.bpl           // end second section
//  BCB5\projects\lib\library.bpi           // start third section
//  BCB5\projects\lib\library.lib           // end third section
//
//  BCB6\lib\library_install.cpp            // start first section
//  BCB6\lib\library_install.h              // end first section
//  BCB6\lib\library_install.obj            // start second section
//  BCB6\projects\bpl\library.bpl           // end second section
//  BCB6\projects\lib\library.bpi           // start third section
//  BCB6\projects\lib\library.lib           // end third section
//

// --- first section
// TDx_Library/TDx_Draw_Library/Source/_Install.*
FileListBox1->Mask = "TDx_Install.H;TDx_Install.CPP";
FileListBox1->Mask = ReplaceText( FileListBox1->Mask, "TDx", Library_Names[lib] );
try {
    FileListBox1->Directory = TDx_Root_Dir + "\\" + Library_Names[lib] + "\\Source";
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find "+Library_Names[lib]+" v"+Library_Versions[lib]+" directory "+TDx_Root_Dir + "\\" + Library_Names[lib] + "\\Source" );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!installing) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];
    AnsiString dest = BCB_Root_Dirs[bcb] + "\\Lib\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = dest;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (FileExists( dest ))
        {
        RESULT( "> Replacing "+dest );
        DeleteFile( dest );
        }
    else
        RESULT( "> Saving "+dest );
    FileCopy( srce, dest, true );
    }


// --- second section
// TDx_Library/Pre-Compiled/BCB1/_Install.OBJ, .BPL's
switch (bcb)
    {
    case BCB1:      FileListBox1->Mask = "TDx_Install.OBJ"; break;
    case BCB3:      FileListBox1->Mask = "TDx_Install.OBJ;TDx.BPL"; break;
    case BCB4:      FileListBox1->Mask = "TDx_Install.OBJ;TDx.BPL"; break;
    case BCB5:      FileListBox1->Mask = "TDx_Install.OBJ;TDx.BPL"; break;
    case BCB6:      FileListBox1->Mask = "TDx_Install.OBJ;TDx.BPL"; break;
    }
FileListBox1->Mask = ReplaceText( FileListBox1->Mask, "TDx", Library_Names[lib] );
try {
    FileListBox1->Directory = TDx_Root_Dir + "\\Pre-Compiled\\" + BCB_Names[bcb];
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find "+Library_Names[lib]+" v"+Library_Versions[lib]+" directory "+TDx_Root_Dir+"\\Pre-Compiled\\"+BCB_Names[bcb] );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!installing) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];
    AnsiString dest;
    switch (bcb)
        {
        case BCB1:  dest = BCB_Root_Dirs[bcb] + "\\Lib\\" + FileListBox1->Items->Strings[file]; break;
        case BCB3:  dest = BCB_Root_Dirs[bcb] + "\\Lib\\" + FileListBox1->Items->Strings[file]; break;
        case BCB4:  dest = BCB_Root_Dirs[bcb] + "\\Projects\\Bpl\\" + FileListBox1->Items->Strings[file]; break;
        case BCB5:  dest = BCB_Root_Dirs[bcb] + "\\Projects\\Bpl\\" + FileListBox1->Items->Strings[file]; break;
        case BCB6:  dest = BCB_Root_Dirs[bcb] + "\\Projects\\Bpl\\" + FileListBox1->Items->Strings[file]; break;
        }

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = dest;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (FileExists( dest ))
        {
        RESULT( "> Replacing "+dest );
        DeleteFile( dest );
        }
    else
        RESULT( "> Saving "+dest );
    FileCopy( srce, dest, true );
    }



// --- third section
switch (bcb)
    {
    case BCB1:      FileListBox1->Mask = ""; break;
    case BCB3:      FileListBox1->Mask = "TDx.BPI;TDx.LIB"; break;
    case BCB4:      FileListBox1->Mask = "TDx.BPI;TDx.LIB"; break;
    case BCB5:      FileListBox1->Mask = "TDx.BPI;TDx.LIB"; break;
    case BCB6:      FileListBox1->Mask = "TDx.BPI;TDx.LIB"; break;
    }
FileListBox1->Mask = ReplaceText( FileListBox1->Mask, "TDx", Library_Names[lib] );
try {
    FileListBox1->Directory = TDx_Root_Dir + "\\Pre-Compiled\\" + BCB_Names[bcb];
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find "+Library_Names[lib]+" v"+Library_Versions[lib]+" directory "+TDx_Root_Dir+"\\Pre-Compiled\\"+BCB_Names[bcb] );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!installing) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];
    AnsiString dest;
    switch (bcb)
        {
        case BCB1:  dest = BCB_Root_Dirs[bcb] + "\\Lib\\" + FileListBox1->Items->Strings[file]; break;
        case BCB3:  dest = BCB_Root_Dirs[bcb] + "\\Lib\\" + FileListBox1->Items->Strings[file]; break;
        case BCB4:  dest = BCB_Root_Dirs[bcb] + "\\Projects\\Lib\\" + FileListBox1->Items->Strings[file]; break;
        case BCB5:  dest = BCB_Root_Dirs[bcb] + "\\Projects\\Lib\\" + FileListBox1->Items->Strings[file]; break;
        case BCB6:  dest = BCB_Root_Dirs[bcb] + "\\Projects\\Lib\\" + FileListBox1->Items->Strings[file]; break;
        }

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = dest;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (FileExists( dest ))
        {
        RESULT( "> Replacing "+dest );
        DeleteFile( dest );
        }
    else
        RESULT( "> Saving "+dest );
    FileCopy( srce, dest, true );
    }
}
//---------------------------------------------------------------------------
void __fastcall TInstallForm::RegisterComponents( int lib, int bcb )
{
TRegistry* registry = new TRegistry();
if (registry==NULL)
    {
    Error( "\
----------------------------------------\n\
Install Components Problem:\n\
----------------------------------------\n\
\n\
The components cannot be installed because there is not enough memory available to access the system registry.\n\
Try reducing the number of applications you currently have open, or buy more memory.\n\
\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should install successfully.\n\
\n\
Error Code: COM_1\n\
\n\
----------------------------------------\n\
" );
    return;
    }
registry->RootKey = HKEY_LOCAL_MACHINE;
registry->LazyWrite = false;

switch (bcb)
    {
    case BCB1:

        ErrorOccurred = false;
        InstallBCB1( lib );
        if (ErrorOccurred)
            {
            Library_Installed[lib][BCB1] = false;
            RESULT( "> ERROR:  BCB1 Install Components Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->WriteString( "BCB1", "True" );
            registry->WriteString( "BCB1_Version", Library_Versions[lib] );
            registry->WriteDate(   "BCB1_First_Installed", Library_First_Installed[lib][bcb] );
            registry->CloseKey();
            Library_Installed[lib][BCB1] = true;
            Library_Installed_Version[lib][BCB1] = Library_Versions[lib];
            RESULT( "> BCB1 Components Installed Successfully!" );
            }
        break;

    case BCB3:

        ErrorOccurred = false;
        InstallBCB3( lib );
        if (ErrorOccurred)
            {
            Library_Installed[lib][BCB3] = false;
            RESULT( "> ERROR:  BCB3 Install Components Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->WriteString( "BCB3", "True" );
            registry->WriteString( "BCB3_Version", Library_Versions[lib] );
            registry->WriteDate(   "BCB3_First_Installed", Library_First_Installed[lib][bcb] );
            registry->CloseKey();
            Library_Installed[lib][BCB3] = true;
            Library_Installed_Version[lib][BCB3] = Library_Versions[lib];
            RESULT( "> BCB3 Components Installed Successfully!" );
            }
        break;

    case BCB4:

        ErrorOccurred = false;
        InstallBCB4( lib );
        if (ErrorOccurred)
            {
            Library_Installed[lib][BCB4] = false;
            RESULT( "> ERROR:  BCB4 Install Components Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->WriteString( "BCB4", "True" );
            registry->WriteString( "BCB4_Version", Library_Versions[lib] );
            registry->WriteDate(   "BCB4_First_Installed", Library_First_Installed[lib][bcb] );
            registry->CloseKey();
            Library_Installed[lib][BCB4] = true;
            Library_Installed_Version[lib][BCB4] = Library_Versions[lib];
            RESULT( "> BCB4 Components Installed Successfully!" );
            }
        break;

    case BCB5:

        ErrorOccurred = false;
        InstallBCB5( lib );
        if (ErrorOccurred)
            {
            Library_Installed[lib][BCB5] = false;
            RESULT( "> ERROR:  BCB5 Install Components Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->WriteString( "BCB5", "True" );
            registry->WriteString( "BCB5_Version", Library_Versions[lib] );
            registry->WriteDate(   "BCB5_First_Installed", Library_First_Installed[lib][bcb] );
            registry->CloseKey();
            Library_Installed[lib][BCB5] = true;
            Library_Installed_Version[lib][BCB5] = Library_Versions[lib];
            RESULT( "> BCB5 Components Installed Successfully!" );
            }
        break;

    case BCB6:

        ErrorOccurred = false;
        InstallBCB6( lib );
        if (ErrorOccurred)
            {
            Library_Installed[lib][BCB6] = false;
            RESULT( "> ERROR:  BCB6 Install Components Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->WriteString( "BCB6", "True" );
            registry->WriteString( "BCB6_Version", Library_Versions[lib] );
            registry->WriteDate(   "BCB6_First_Installed", Library_First_Installed[lib][bcb] );
            registry->CloseKey();
            Library_Installed[lib][BCB5] = true;
            Library_Installed_Version[lib][BCB5] = Library_Versions[lib];
            RESULT( "> BCB6 Components Installed Successfully!" );
            }
        break;

    }
delete registry;
Application->HandleMessage();
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#include "shlobj.hpp"

//---------------------------------------------------------------------------
TStrings* ElementStrings( AnsiString SourceText, AnsiString Delimeter )
{
if (SourceText=="") return NULL;
TStrings* strings = new TStringList();
if (strings==NULL) return NULL;

/*
int pos;
int length = Delimeter.Length();
while (pos = SourceText.Pos(Delimeter))
    {
    strings->Add( SourceText.SubString(1,pos-1) );
    SourceText = SourceText.SubString(pos+length,SourceText.Length());
    }
return strings;
*/

char* source = SourceText.c_str();
char* delimeter = Delimeter.c_str();
int len = SourceText.Length();
int delimeter_length = Delimeter.Length();
int delimeter_count = 0;
AnsiString normal_text = "";
AnsiString possibly_normal_text = "";
for (int i=0;i<len;i++)
    {
    if (*source == *delimeter)  // if we found a delimeter
        {
        *delimeter++;           // get ready for the next one
        delimeter_count++;      // and remember how many we've completed
        if (delimeter_count==delimeter_length) // if we found a completed delimeter
            {
            // save the our built-up string
            strings->Add(normal_text);
            normal_text = "";
            possibly_normal_text = "";
            }
        else
            possibly_normal_text += *source;
        }
    else
        {
        normal_text += possibly_normal_text;
        possibly_normal_text = "";
        normal_text += *source;
        delimeter = Delimeter.c_str();
        delimeter_count = 0;
        }
    *source++;
    }
if (normal_text!="") strings->Add(normal_text);
return strings;
}
//---------------------------------------------------------------------------
AnsiString  ElementString( AnsiString SourceText, AnsiString Delimeter, int ElementNumber )
{
TStrings* elems = ElementStrings( SourceText, Delimeter );
if (elems == NULL) return "";
AnsiString str = "";
if ((ElementNumber>=0) && (ElementNumber<=elems->Count-1))
    str = elems->Strings[ElementNumber];
delete elems;
return str;
}
//---------------------------------------------------------------------------
int ElementCount( AnsiString SourceText, AnsiString Delimeter )
{
TStrings* elems = ElementStrings( SourceText, Delimeter );
if (elems == NULL) return 0;
int count = elems->Count;
delete elems;
return count;
}
//---------------------------------------------------------------------------
AnsiString  ReplaceText( AnsiString SourceText,
                         AnsiString ReplaceWhat,
                         AnsiString WithWhat )
{
if (SourceText=="") return "";
if (ReplaceWhat=="") return SourceText;
if (ReplaceWhat==WithWhat) return SourceText;

AnsiString return_string = "";
int iter = 0;
int srce_pos = 1;
while ((srce_pos != 0) && (iter<1000000))
    {
    srce_pos = SourceText.Pos(  ReplaceWhat );
    if (srce_pos!=0)
        {
        return_string = return_string + SourceText.SubString( 1, srce_pos-1 ) + WithWhat;
        SourceText.Delete( 1, srce_pos-1 + ReplaceWhat.Length() );
        }
    else
        return_string = return_string + SourceText;
    iter++;
    }
return return_string;
}
//---------------------------------------------------------------------------
bool FileCopy( AnsiString SourcePath, AnsiString DestPath, bool ReplaceExistingFile)
{
BOOL replace = (ReplaceExistingFile) ? TRUE : FALSE;
int result = CopyFile( SourcePath.c_str(), DestPath.c_str(), replace );
return (result!=0);
}
//---------------------------------------------------------------------------
AnsiString GetRootPath( AnsiString pDirectory )
{
if (pDirectory.SubString(1,1)=="\\")
    return GetCurrentDir().SubString(1,3);

if (pDirectory.SubString(1,2)==":\\")
    return GetCurrentDir().SubString(1,3);

if (pDirectory.SubString(2,2)==":\\")
    return pDirectory.SubString(1,3);

return pDirectory;
}
//---------------------------------------------------------------------------
AnsiString GetFolderRelativePath( AnsiString pDirectory )
{
if (pDirectory.SubString(1,1)=="\\")
    return pDirectory.SubString(2,pDirectory.Length()-1);

if (pDirectory.SubString(1,2)==":\\")
    return pDirectory.SubString(3,pDirectory.Length()-2);

if (pDirectory.SubString(2,2)==":\\")
    return pDirectory.SubString(4,pDirectory.Length()-3);

return pDirectory;
}
//---------------------------------------------------------------------------
AnsiString SystemRootDirectory()
{
TRegistry* registry = new TRegistry();
if (registry==NULL) return "";

registry->RootKey = HKEY_LOCAL_MACHINE;
registry->OpenKey( "\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion", false );
AnsiString system_root = registry->ReadString( "SystemRoot" );
registry->CloseKey();
delete registry;
return system_root;
}
//---------------------------------------------------------------------------
bool CreateBigDir( AnsiString pDirectory )
{
// determine the root drive of the new directory
AnsiString new_root = GetRootPath(pDirectory);
AnsiString new_folder = GetFolderRelativePath(pDirectory);

// split up directories
TStrings* dirs = ElementStrings( new_folder, "\\" );
if (dirs==NULL) return false;

// save current directory
AnsiString original_directory = GetCurrentDir();

// change to relevant drive's root directory
if (!SetCurrentDir( new_root )) return false;

// traverse the directory and create each subdir as required
bool ok = true;
for (int i=0;i<dirs->Count;i++)
    {
    // if directory does not already exist, create it
    if (!DirectoryExists( dirs->Strings[i])) CreateDir( dirs->Strings[i] );

    // change to the newly created directory
    if (DirectoryExists( dirs->Strings[i] )) SetCurrentDir( dirs->Strings[i] );
    else
        // abort if not created for any reason.
        {
        ok = false;
        break;
        }
    }

// release directory strings
delete dirs;

// restore original directory
SetCurrentDir( original_directory );

// return status
return ok;
}
//---------------------------------------------------------------------------
// eg.  pFolderName = TDx_Library
//      pStartMenuLocation = Programs\TDx_Library
bool CreateStartMenuFolder( AnsiString pStartMenuLocation )
{
if (pStartMenuLocation=="") return false;

// get system folder eg. C:\Windows
AnsiString system_root = SystemRootDirectory();
if (system_root=="") return false;

// make it the start menu eg. C:\Windows\Start Menu
AnsiString start_menu_root = system_root + "\\Start Menu";

// determine entire string. ie. C:\Windows\Start Menu\Programs\TDx_Library
AnsiString new_folder = start_menu_root;
if (pStartMenuLocation.SubString(1,1)!="\\") new_folder = new_folder + "\\";
new_folder = new_folder + pStartMenuLocation;

// create the new folder
return CreateBigDir( new_folder );
}
//---------------------------------------------------------------------------
bool CreateStartMenuLink(AnsiString pStartMenuLocation, AnsiString pLinkName, AnsiString pFileName )
{
AnsiString system_root = SystemRootDirectory();
if (system_root=="") return false;
/*
AnsiString start_menu_root = system_root + "\\Start Menu";
AnsiString link_location = start_menu_root;
if (pStartMenuLocation.SubString(1,1)!="\\") link_location = link_location + "\\";
link_location = link_location + pStartMenuLocation;
link_location = link_location + "\\" + pLinkName;
if (link_location.UpperCase().Pos(".LNK")==0) link_location = link_location + ".LNK";

// initialize COM
CoInitialize( NULL );

// Get a pointer to the IShellLink interface.
IShellLink* shell_link;
HRESULT result = CoCreateInstance( (_GUID) CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER | CLSCTX_LOCAL_SERVER | CLSCTX_INPROC_HANDLER, (_GUID) IID_IShellLink, (void**) &shell_link );
if (SUCCEEDED(result))
    {
    // Query IShellLink for the IPersistFile interface
    // used to save the shortcut in persistent storage.
    IPersistFile* persist_file;
    result = shell_link->QueryInterface( (_GUID)IID_IPersistFile, (void**) &persist_file );
    if (SUCCEEDED(result))
        {
        // set the path to the shortcut target, add a description
        AnsiString filename_directory = ExtractFileDir( pFileName );

        shell_link->SetPath((LPCSTR) pFileName.c_str());
        shell_link->SetDescription((LPSTR) pLinkName.c_str() );
        shell_link->SetWorkingDirectory((LPCSTR) filename_directory.c_str());

        // convert the ansi string to a wide char string
        WORD wide_string[MAX_PATH];
        MultiByteToWideChar(CP_ACP, 0, (LPSTR) link_location.c_str(), -1, (wchar_t*) wide_string, MAX_PATH );

        // Save the link by calling IPersistFile::Save.
        persist_file->Save( (wchar_t*) wide_string, TRUE );

        // release IPersistFile
        persist_file->Release();
        }

    // Release IShellLink
    shell_link->Release();
    }
else
    ShowMessage("FAILED ON SHELL LINK");

// disable COM
CoUninitialize();
*/
return true;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void __fastcall TInstallForm::RegisterStartMenu( int lib, int bcb )
{
/*
CreateStartMenuFolder( "Programs\\TDx_Library" );
CreateStartMenuFolder( "Programs\\TDx_Library\\Install" );
CreateStartMenuFolder( "Programs\\TDx_Library\\Examples" );
CreateStartMenuFolder( "Programs\\TDx_Library\\"+Library_Name );
CreateStartMenuFolder( "Programs\\TDx_Library\\"+Library_Name+"\\Build" );
CreateStartMenuFolder( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB1" );
CreateStartMenuFolder( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB3" );
CreateStartMenuFolder( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB4" );
CreateStartMenuFolder( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB5" );
CreateStartMenuFolder( "Programs\\TDx_Library\\"+Library_Name+"\\Help" );
CreateStartMenuFolder( "Programs\\TDx_Library\\"+Library_Name+"\\Html" );
CreateStartMenuFolder( "Programs\\TDx_Library\\"+Library_Name+"\\Examples" );
CreateStartMenuFolder( "Programs\\TDx_Library\\"+Library_Name+"\\Source" );

CreateStartMenuLink( "Programs\\TDx_Library\\Install","TDx_Library_Install.exe",TDx_Root_Dir+"\\"+Install\\TDx_Library_Install.exe" );

CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"","Readme.txt",TDx_Root_Dir+"\\"+Library_Name+"\\Readme.TXT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"","Licence.txt",TDx_Root_Dir+"\\"+Library_Name+"\\Licence.TXT" );

CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build","Build_ALL.bat",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\Build_ALL.BAT" );

CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB1","Build.bat",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB1\\Build.BAT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB1","Before_Build.bat",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB1\\Before_Build.BAT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB1","After_Build.bat",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB1\\After_Build.BAT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB1","Readme.txt",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB1\\Readme.TXT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB1",""+Library_Name+"_Build.mak",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB1\\"+Library_Name+"Build.MAK" );

CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB3","Build.bat",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB3\\Build.BAT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB3","Before_Build.bat",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB3\\Before_Build.BAT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB3","After_Build.bat",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB3\\After_Build.BAT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB3","Readme.txt",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB3\\Readme.TXT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB3",""+Library_Name+"_Build.bpk",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB3\\"+Library_Name+"Build.BPK" );

CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB4","Build.bat",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB4\\Build.BAT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB4","Before_Build.bat",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB4\\Before_Build.BAT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB4","After_Build.bat",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB4\\After_Build.BAT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB4","Readme.txt",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB4\\Readme.TXT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB4",""+Library_Name+"_Build.bpk",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB4\\"+Library_Name+"Build.BPK" );

CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB5","Build.bat",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB5\\Build.BAT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB5","Before_Build.bat",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB5\\Before_Build.BAT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB5","After_Build.bat",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB5\\After_Build.BAT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB5","Readme.txt",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB5\\Readme.TXT" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Build\\BCB5",""+Library_Name+"_Build.bpk",TDx_Root_Dir+"\\"+Library_Name+"\\Build\\BCB5\\"+Library_Name+"Build.BPK" );

CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Help",""+Library_Name+".hlp",TDx_Root_Dir+"\\"+Library_Name+"\\Help\\"+Library_Name+".HLP" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Html","Welcome.htm",TDx_Root_Dir+"\\"+Library_Name+"\\Html\\Welcome.htm" );

CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Install",""+Library_Name+"_Install.exe",TDx_Root_Dir+"\\"+Library_Name+"\\Install\\"+Library_Name+"_Install.exe" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Examples\\Blank Template\\BCB1","Template.mak",TDx_Root_Dir+"\\"+Library_Name+"\\Examples\\Blank Template\\BCB1\\Template.MAK" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Examples\\Blank Template\\BCB3","Template.bpr",TDx_Root_Dir+"\\"+Library_Name+"\\Examples\\Blank Template\\BCB3\\Template.BPR" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Examples\\Blank Template\\BCB4","Template.bpr",TDx_Root_Dir+"\\"+Library_Name+"\\Examples\\Blank Template\\BCB4\\Template.BPR" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Examples\\Blank Template\\BCB5","Template.bpr",TDx_Root_Dir+"\\"+Library_Name+"\\Examples\\Blank Template\\BCB5\\Template.BPR" );
CreateStartMenuLink( "Programs\\TDx_Library\\"+Library_Name+"\\Examples\\Blank Template\\Source","Template.cpp",TDx_Root_Dir+"\\"+Library_Name+"\\Examples\\Blank Template\\Source\\Template.CPP" );
*/
}
//---------------------------------------------------------------------------
void __fastcall TInstallForm::RegisterComponentHelp( int lib, int bcb )
{
TRegistry* registry = new TRegistry();
if (registry==NULL)
    {
    Error( "\
--------------------------------------------\n\
Install Component Help Problem:\n\
--------------------------------------------\n\
\n\
The help files cannot be installed because there is not enough memory available to access the system registry.\n\
Try reducing the number of applications you currently have open, or buy more memory.\n\
\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should install successfully.\n\
\n\
Error Code: HELP_1\n\
\n\
--------------------------------------------\n\
" );
    return;
    }
registry->RootKey = HKEY_LOCAL_MACHINE;
registry->LazyWrite = false;

switch (bcb)
    {
    case BCB1:

        ErrorOccurred = false;
        InstallBCB1Help( lib );
        if (ErrorOccurred)
            {
            Library_Installed_Help[lib][BCB1] = false;
            RESULT( "> ERROR:  BCB1 Install Help Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->WriteString( "BCB1_Help", "True" );
            registry->CloseKey();
            Library_Installed_Help[lib][BCB1] = true;
            RESULT( "> BCB1 Help Installed Successfully!" );
            }
        break;

    case BCB3:

        ErrorOccurred = false;
        InstallBCB3Help( lib );
        if (ErrorOccurred)
            {
            Library_Installed_Help[lib][BCB3] = false;
            RESULT( "> ERROR:  BCB3 Install Help Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->WriteString( "BCB3_Help", "True" );
            registry->CloseKey();
            Library_Installed_Help[lib][BCB3] = true;
            RESULT( "> BCB3 Help Installed Successfully!" );
            }
        break;

    case BCB4:

        ErrorOccurred = false;
        InstallBCB4Help( lib );
        if (ErrorOccurred)
            {
            Library_Installed_Help[lib][BCB4] = false;
            RESULT( "> ERROR:  BCB4 Install Help Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->WriteString( "BCB4_Help", "True" );
            registry->CloseKey();
            Library_Installed_Help[lib][BCB4] = true;
            RESULT( "> BCB4 Help Installed Successfully!" );
            }
        break;

    case BCB5:

        ErrorOccurred = false;
        InstallBCB5Help( lib );
        if (ErrorOccurred)
            {
            Library_Installed_Help[lib][BCB5] = false;
            RESULT( "> ERROR:  BCB5 Install Help Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->WriteString( "BCB5_Help", "True" );
            registry->CloseKey();
            Library_Installed_Help[lib][BCB5] = true;
            RESULT( "> BCB5 Help Installed Successfully!" );
            }
        break;

    case BCB6:

        ErrorOccurred = false;
        InstallBCB6Help( lib );
        if (ErrorOccurred)
            {
            Library_Installed_Help[lib][BCB6] = false;
            RESULT( "> ERROR:  BCB6 Install Help Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->WriteString( "BCB6_Help", "True" );
            registry->CloseKey();
            Library_Installed_Help[lib][BCB6] = true;
            RESULT( "> BCB6 Help Installed Successfully!" );
            }
        break;

    }
delete registry;
Application->HandleMessage();
}
//---------------------------------------------------------------------------
#include "InstallBCB1.inc"
#include "InstallBCB3.inc"
#include "InstallBCB4.inc"
#include "InstallBCB5.inc"
#include "InstallBCB6.inc"
#include "InstallBCB1Help.inc"
#include "InstallBCB3Help.inc"
#include "InstallBCB4Help.inc"
#include "InstallBCB5Help.inc"
#include "InstallBCB6Help.inc"
//---------------------------------------------------------------------------

void __fastcall TInstallForm::SpeedButton1Click(TObject *Sender)
{
TClipboard* clip = Clipboard();
clip->AsText = Results->Items->Text;
CopiedLabel->Caption = "( copied )";
CopiedLabel->Update();
Application->HandleMessage();
}
//---------------------------------------------------------------------------

void __fastcall TInstallForm::BCBTools_AnimMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
MainForm->BCBTools_AnimMouseMove( this, Shift, X, Y );
}
//---------------------------------------------------------------------------


void __fastcall TInstallForm::BCBToolsClick(TObject *Sender)
{
MainForm->BCBTools->OnClick(this);
}
//---------------------------------------------------------------------------


void __fastcall TInstallForm::FormPaint(TObject *Sender)
{
Canvas->Brush->Color = clBlack;
Canvas->Brush->Style = bsSolid;
Canvas->FrameRect( ClientRect );
}
//---------------------------------------------------------------------------
void __fastcall TInstallForm::UninstallClick(TObject *Sender)
{
MainForm->Prev->Play(0,0);
}
//---------------------------------------------------------------------------
void __fastcall TInstallForm::CancelClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TInstallForm::InstallMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
if (!Active) return;
TComponent* sender = (TComponent*) Sender;
if (sender==NULL) return;

// if there is an image that is already Highlighted, un-highlight it now.
if ((sender->Name!="Install") && (Highlighted=="Install")) { InstallImage->Picture = MainForm->Lo->Picture; Install->Font->Color = clNavy; }
else if ((sender->Name!="Prev") && (Highlighted=="Prev")) { PrevImage->Picture = MainForm->Lo->Picture; Prev->Font->Color = clNavy; }
else if ((sender->Name!="Exit") && (Highlighted=="Exit")) { ExitImage->Picture = MainForm->Lo->Picture; Exit->Font->Color = clNavy; }

// if the image is not already Highlighted, but now needs to be Highlighted....
if ((sender->Name=="Install") && (Highlighted != "Install")) { InstallImage->Picture = MainForm->Hi->Picture; Install->Font->Color = clWhite; if (MainForm->Pop->Playing) MainForm->Pop->Stop(); MainForm->Pop->Play(0,0); }
else if ((sender->Name=="Prev") && (Highlighted!="Prev")) { PrevImage->Picture = MainForm->Hi->Picture; Prev->Font->Color = clWhite; if (MainForm->Pop->Playing) MainForm->Pop->Stop(); MainForm->Pop->Play(0,0); }
else if ((sender->Name=="Exit") && (Highlighted!="Exit")) { ExitImage->Picture = MainForm->Hi->Picture; Exit->Font->Color = clWhite; if (MainForm->Pop->Playing) MainForm->Pop->Stop(); MainForm->Pop->Play(0,0); }

// remember which button is Highlighted
Highlighted = sender->Name;
}
//---------------------------------------------------------------------------
void __fastcall TInstallForm::PrevClick(TObject *Sender)
{
if (!installing)
    {
    if (MainForm->Prev->Playing) MainForm->Prev->Stop();
    MainForm->Prev->Play(0,0);
    MainForm->GetCurrentInstallStatus();
    Close();
    }
}
//---------------------------------------------------------------------------
void __fastcall TInstallForm::FormShow(TObject *Sender)
{
Top = MainForm->Top;
Left = MainForm->Left;
Width = MainForm->Width;
Height = MainForm->Height;
}
//---------------------------------------------------------------------------



void __fastcall TInstallForm::TitleClick(TObject *Sender)
{
MainForm->Title->OnClick(this);
}
//---------------------------------------------------------------------------

