//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "TUnInstallForm.h"
#include "TMainForm.h"
#include "TErrorForm.h"
#include "MiscellaneousFunctions.h"
#include "TInstallSetupForm.h"
#include "registry.hpp"
#include "stdio.h"
#include "fcntl.h"
#include "io.h"
#include "dir.h"
#include "sys\stat.h"
#include "clipbrd.hpp"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "TDx_Sound_Library_Install"
#pragma link "TDx_Sound"
#pragma link "TDx_SoundBuffer"
#pragma link "TDSBufferDesc"
#pragma link "PERFGRAP"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
TUnInstallForm *UnInstallForm;
//---------------------------------------------------------------------------
#define ACTION( color, str )        {\
                                    Action->Font->Color = (color);\
                                    Action->Caption = (str);\
                                    Action->Update();\
                                    }\

#define RESULT( str )               {\
                                    try {\
                                        Results->Items->Add( (str) );\
                                        }\
                                    catch (Exception &e)\
                                        {\
                                        try {\
                                            Results->Items->Clear();\
                                            Results->Items->Add( (str) );\
                                            }\
                                        catch (Exception &e)\
                                            {\
                                            }\
                                        }\
                                    if (Results->Items->Count>0) Results->ItemIndex = Results->Items->Count-1;\
                                    Results->Update();\
                                    }\

//---------------------------------------------------------------------------
struct TPackItHeader
    {
    word        Version;
    dword       TotalFiles;
    TDateTime   DateCreated;
    };
//---------------------------------------------------------------------------
struct TPackItFileHeader
    {
    dword       FilePos;
    char        FileName[1024];
    dword       FileSize;
    };
//---------------------------------------------------------------------------
void __fastcall TUnInstallForm::DeleteBasedOnPackIt( int lib )
{
AnsiString filename = Install_Directory+"\\"+Library_Names[lib].UpperCase()+".PIT";

// --------------------
// extract files...
// --------------------
TPackItHeader header;
TPackItFileHeader fileheader;

int file = open(filename.c_str(),O_RDWR|O_BINARY);
if (file == -1)
    {
    Error( "\
--------------------------------\n\
.PIT File Removal Problem:\n\
--------------------------------\n\
\n\
This install program normally removes all files based on data in the accompanying .PIT file :\n\
\n\
\n  \""+filename+"\"\n\
\n\
This file cannot be read by the install application.\n\
It is possible it either does not exist or is marked with no read access.\n\
\n\
Try replacing the file with a different copy of the "+Library_Names[lib].UpperCase()+".PIT"+" file.\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should uninstall successfully.\n\
\n\
Error Code: RPIT_1\n\
\n\
---------------------------\n\
" );
    return;
    }

// read PackIt file header (to get total # of files)
int error = read( file, &header, sizeof(TPackItHeader) );
if (error == -1)
    {
    close(file);
    Error( "\
--------------------------------\n\
.PIT File Removal Problem:\n\
--------------------------------\n\
\n\
This install program normally removes all files based on data in the accompanying .PIT file :\n\
\n\
\n  \""+filename+"\"\n\
\n\
This file exists at the expected location but appears to be corrupted and as such will no longer function correctly.\n\
\n\
Try replacing the file with a different copy of the "+Library_Names[lib].UpperCase()+".PIT"+" file.\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should install successfully.\n\
\n\
Error Code: RPIT_2\n\
\n\
---------------------------\n\
" );
    return;
    }

// save the current directory so we can change back later
AnsiString original_dir = GetCurrentDir();

// extract all PackIt files...
FileProgress->Min = 0;
FileProgress->Max = header.TotalFiles;
FileProgress->Position = 0;
FileProgress->Update();

for (dword i=0;i<header.TotalFiles;i++)
    {
    Application->HandleMessage();
    if (!deleting) break;

    // read file info
    read( file, &fileheader, sizeof(TPackItFileHeader) );

    // all files come in as :-
    //  eg.
    //      D:\TDx_Library\*.*
    // need to be
    //      O:\TDx_Root_Dir\TDx_Library\*.*

    AnsiString old_filename = fileheader.FileName;
    AnsiString new_filename = ReplaceText( old_filename, "D:\\TDx_Library", TDx_Root_Dir );
    AnsiString just_filename = ExtractFileName( new_filename );

    // check if TDx_Library_Defns + TDx_Library_Functions are still required
    bool other_libs_still_installed = false;
    
    AnsiString temp = just_filename.LowerCase();
    if ((temp=="tdx_library_defns.cpp") ||
        (temp=="tdx_library_defns.h") ||
        (temp=="tdx_library_defns.obj") ||
        (temp=="tdx_library_functions.cpp") ||
        (temp=="tdx_library_functions.h") ||
        (temp=="tdx_library_functions.obj"))
        {
        for (int i=0;i<NUM_LIBS;i++)
            for (int bcb=0;bcb<NUM_BCB;bcb++)
                {
                if (Library_Installed[i][bcb])
                    {
                    other_libs_still_installed = true;
                    i = NUM_LIBS;
                    break;
                    }
                }
        }

    File->Caption = "[ "+just_filename+" ]";
    File->Update();
    FilePath->Caption = new_filename;
    FilePath->Update();
    FileCount->Caption = IntToStr(i+1) + " of " + IntToStr(header.TotalFiles);
    FileCount->Update();
    FileProgress->Position = i;
    FileProgress->Update();

    if (FileExists(new_filename) && !other_libs_still_installed)
        {
        RESULT( "> Deleting "+new_filename );

        // if for some reason, it is read-only.... change it to read-write
        chmod( new_filename.c_str(), S_IREAD | S_IWRITE );
        DeleteFile(new_filename);

        // if it's still there, make error noise
        if (FileExists(new_filename))
            {
            close(file);
            Error( "\
--------------------------------\n\
.PIT File Removal Problem:\n\
--------------------------------\n\
\n\
This install program normally deletes files that also exist in the accompanying .PIT file.\n\
Unfortunately one of the files could not be deleted successfully :\n\
\n\
  File: \""+new_filename+"\"\n\
\n\
It may be that the file exists but cannot be deleted, try removing the old file manually.\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should uninstall successfully.\n\
\n\
Error Code: RPIT_3\n\
\n\
---------------------------\n\
" );
            break;
            }
        }

    // skip entire file
    lseek( file, fileheader.FileSize, SEEK_CUR );
    }
close(file);
}
//---------------------------------------------------------------------------
__fastcall TUnInstallForm::TUnInstallForm(TComponent* Owner)
        : TForm(Owner)
{
deleting = false;
RemoveComplete = false;
}
//---------------------------------------------------------------------------
void __fastcall TUnInstallForm::ExitClick(TObject *Sender)
{
if (MainForm->Goodbye->Play(0,0))
while (MainForm->Goodbye->Playing) ;  // wait for end playing
Application->Terminate();
}
//---------------------------------------------------------------------------

void __fastcall TUnInstallForm::InstallClick(TObject *Sender)
{
if (MainForm->Next->Playing) MainForm->Next->Stop();
MainForm->Next->Play(0,0);

if (deleting)
    {
    deleting = false;

    if (Action->Caption!="> Remove Complete") ACTION( clMaroon, "> Remove Aborted" );
    RESULT( "------------------------------------------------------------------------------------------------" );
    RESULT( Action->Caption );
    RESULT( "------------------------------------------------------------------------------------------------" );

    try {
        AnsiString name = Install_Directory + "\\TDx_Uninstall.log";
        if (FileExists( name )) DeleteFile( name );
        Results->Items->SaveToFile( name );
        RESULT( "> Saved remove results to "+name );
        }
    catch (...)
        {
        }

    MainForm->InstallWav->Stop();

    Install->Caption = "> start";
    Install->Hint = "Begin the removal.";
    Install->Update();
    Installing->Caption = "-";
    Installing->Hint = "Ready to remove";
    Installing->Update();

    Actions->Visible = true;
    Install->Visible = true;
    InstallImage->Visible = true;
    Prev->Visible = true;
    PrevImage->Visible = true;
    Exit->Visible = true;
    ExitImage->Visible = true;
    ExitHere->Visible = true;
    }
else
    {
    RemoveComplete = false;
    MainForm->InstallWav->Play(0,DSBPLAY_LOOPING);

    TRegistry* registry = new TRegistry();
    if (registry==NULL)
        {
        Error( "\
------------------\n\
Remove Problem:\n\
------------------\n\
\n\
The components cannot be removed because there is not enough memory available to access the system registry.\n\
Try reducing the number of applications you currently have open, or buy more memory.\n\
\n\
Once the problem is resolved the TDx_Library should remove successfully.\n\
\n\
Error Code: REMOVE_1\n\
\n\
---------------------------\n\
" );
        return;
        }
    registry->RootKey = HKEY_CURRENT_USER;
    registry->LazyWrite = false;
    TRegKeyInfo info;

    deleting = true;
    Results->Items->Clear();
    CopiedLabel->Caption = "( Copy text to Clipboard )";
    CopiedLabel->Visible = false;
    SpeedButton1->Visible = false;
    Application->HandleMessage();

    ACTION( clGreen, "> Remove Started" );
    RESULT( "------------------------------------------------------------------------------------------------" );
    RESULT( Action->Caption );

    Install->Caption = "[] STOP";
    Install->Hint = "Stop the removal [NOT RECOMMENDED] ";
    Install->Update();
    Installing->Caption = "-";
    Installing->Hint = "Currently removing";
    Installing->Update();

    Prev->Visible = false;
    PrevImage->Visible = false;
    Exit->Visible = false;
    ExitImage->Visible = false;
    ExitHere->Visible = false;

    // remove procedure
    for (int lib=0;lib<NUM_LIBS;lib++)
        {
        Application->HandleMessage();
        if (!deleting) break;

        bool remove_now = false;
        switch (lib)
            {
            case DRAW:   remove_now = true; break;
            case SOUND:  remove_now = true; break;
            case INPUT:  remove_now = true; break;
            case PLAY:   remove_now = true; break;
            case D3I:    remove_now = true; break;
            }

        if (remove_now)
            {
            RESULT( "------------------------------------------------------------------------------------------------" );
            ACTION( clGreen, "> Removing "+Library_Names[lib] );
            RESULT( Action->Caption );
            RESULT( "------------------------------------------------------------------------------------------------" );

            Installing->Caption = Library_Names[lib];
            Installing->Hint = "Removing "+Library_Names[lib];
            Installing->Update();
            }

        Lib->Caption = "[ "+Library_Names[lib]+" ]";
        Lib->Update();
        LibCount->Caption = IntToStr(lib+1) + " of " + IntToStr(NUM_LIBS);
        LibCount->Update();
        LibProgress->Position = lib+1;
        LibProgress->Update();

        BCB->Caption = "-";
        BCB->Update();
        BCBCount->Caption = "-";
        BCBCount->Update();
        BCBProgress->Position = 0;
        BCBProgress->Update();

        File->Caption = "-";
        File->Update();
        FilePath->Caption = "-";
        FilePath->Update();
        FileCount->Caption = "-";
        FileCount->Update();
        FileProgress->Position = 0;
        FileProgress->Update();

        // remove for all BCB's
        for (int bcb=0;bcb<NUM_BCB;bcb++)
            {
            Application->HandleMessage();
            if (!deleting) break;

            BCB->Caption = "[ "+BCB_Names[bcb]+" ]";
            BCB->Update();
            BCBCount->Caption = IntToStr(bcb+1) + " of " + IntToStr(NUM_BCB);
            BCBCount->Update();
            BCBProgress->Position = bcb+1;
            BCBProgress->Update();

            // UnInstall Component Help Files
            if (!deleting) break;
            if (remove_now && BCB_Exists[bcb])
                {
                RESULT( "------------------------------------------------------------------------------------------------" );
                ACTION( clGreen, "> Removing "+Library_Names[lib]+" Help Files from "+BCB_Names[bcb]+":" );
                RESULT( Action->Caption );
                RESULT( "------------------------------------------------------------------------------------------------" );

                UnRegisterComponentHelp( lib, bcb );
                }

            // UnInstall Components
            if (!deleting) break;
            if (remove_now && BCB_Exists[bcb])
                {
                RESULT( "------------------------------------------------------------------------------------------------" );
                ACTION( clGreen, "> Removing "+Library_Names[lib]+" Components from "+BCB_Names[bcb]+":" );
                RESULT( Action->Caption );
                RESULT( "------------------------------------------------------------------------------------------------" );

                UnRegisterComponents( lib, bcb );
                }

            // Remove TDx Source Code
            if (!deleting) break;
            if (remove_now && BCB_Exists[bcb])
                {
                RESULT( "------------------------------------------------------------------------------------------------" );
                ACTION( clGreen, "> Remove "+Library_Names[lib]+" Source Code from "+BCB_Names[bcb] );
                RESULT( Action->Caption );
                RESULT( "------------------------------------------------------------------------------------------------" );

                DeleteSourceCodeFiles( lib, bcb );
                }

            // Remove TDx Project Files
            if (!deleting) break;
            if (remove_now && BCB_Exists[bcb])
                {
                RESULT( "------------------------------------------------------------------------------------------------" );
                ACTION( clGreen, "> Remove "+Library_Names[lib]+" Project Files from "+BCB_Names[bcb] );
                RESULT( Action->Caption );
                RESULT( "------------------------------------------------------------------------------------------------" );

                DeleteProjectFiles( lib, bcb );
                }

            // Remove TDx Lib Files
            if (!deleting) break;
            if (remove_now && BCB_Exists[bcb])
                {
                RESULT( "------------------------------------------------------------------------------------------------" );
                ACTION( clGreen, "> Remove "+Library_Names[lib]+" Library Files from "+BCB_Names[bcb] );
                RESULT( Action->Caption );
                RESULT( "------------------------------------------------------------------------------------------------" );

                DeleteLibFiles( lib, bcb );
                }

            // Remove TDx Include Files
            if (remove_now && BCB_Exists[bcb])
                {
                RESULT( "------------------------------------------------------------------------------------------------" );
                ACTION( clGreen, "> Remove "+Library_Names[lib]+" Include Files from "+BCB_Names[bcb] );
                RESULT( Action->Caption );
                RESULT( "------------------------------------------------------------------------------------------------" );

                DeleteIncludeFiles( lib, bcb );
                }
            }

        // Delete Files
        if (!deleting) break;
        if (remove_now)
            {
            RESULT( "------------------------------------------------------------------------------------------------" );
            ACTION( clGreen, "> Deleting "+Library_Names[lib]+" Files:" );
            RESULT( Action->Caption );
            RESULT( "------------------------------------------------------------------------------------------------" );

            DeleteBasedOnPackIt( lib );

            RESULT( "> Removing Unused Directories" );
            if (RemoveDir( TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Build\\BCB1" ))  RESULT( "> Removed "+TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Build\\BCB1" );
            if (RemoveDir( TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Build\\BCB3" ))  RESULT( "> Removed "+TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Build\\BCB3" );
            if (RemoveDir( TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Build\\BCB4" ))  RESULT( "> Removed "+TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Build\\BCB4" );
            if (RemoveDir( TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Build\\BCB5" ))  RESULT( "> Removed "+TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Build\\BCB5" );
            if (RemoveDir( TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Build\\BCB6" ))  RESULT( "> Removed "+TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Build\\BCB6" );
            if (RemoveDir( TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Build" ))        RESULT( "> Removed "+TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Build" );
            if (RemoveDir( TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Help" ))         RESULT( "> Removed "+TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Help" );
            if (RemoveDir( TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Html" ))         RESULT( "> Removed "+TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Html" );
            if (RemoveDir( TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Source" ))       RESULT( "> Removed "+TDx_Root_Dir+"\\"+Library_Names[lib]+"\\Source" );
            if (RemoveDir( TDx_Root_Dir+"\\"+Library_Names[lib] ))                  RESULT( "> Removed "+TDx_Root_Dir+"\\"+Library_Names[lib] );
            if (RemoveDir( TDx_Root_Dir+"\\Pre-Compiled\\BCB1" ))                   RESULT( "> Removed "+TDx_Root_Dir+"\\Pre-Compiled\\BCB1" );
            if (RemoveDir( TDx_Root_Dir+"\\Pre-Compiled\\BCB3" ))                   RESULT( "> Removed "+TDx_Root_Dir+"\\Pre-Compiled\\BCB3" );
            if (RemoveDir( TDx_Root_Dir+"\\Pre-Compiled\\BCB4" ))                   RESULT( "> Removed "+TDx_Root_Dir+"\\Pre-Compiled\\BCB4" );
            if (RemoveDir( TDx_Root_Dir+"\\Pre-Compiled\\BCB5" ))                   RESULT( "> Removed "+TDx_Root_Dir+"\\Pre-Compiled\\BCB5" );
            if (RemoveDir( TDx_Root_Dir+"\\Pre-Compiled\\BCB6" ))                   RESULT( "> Removed "+TDx_Root_Dir+"\\Pre-Compiled\\BCB6" );
            if (RemoveDir( TDx_Root_Dir+"\\Pre-Compiled" ))                         RESULT( "> Removed "+TDx_Root_Dir+"\\Pre-Compiled" );
            if (RemoveDir( TDx_Root_Dir+"\\Source" ))                               RESULT( "> Removed "+TDx_Root_Dir+"\\Source" );
            if (RemoveDir( TDx_Root_Dir+"\\Html" ))                                 RESULT( "> Removed "+TDx_Root_Dir+"\\Html" );
            if (RemoveDir( TDx_Root_Dir ))                                          RESULT( "> Removed "+TDx_Root_Dir );
            }
        }

    // Restore DirectX sdk files for all selected bcb versions
    if (true)
        {
        bool copied_libs[NUM_BCB];
        bool copied_includes[NUM_BCB];
        for (int bcb=0;bcb<NUM_BCB;bcb++)
            {
            copied_libs[bcb] = false;
            copied_includes[bcb] = false;
            }

        BCB->Caption = "-";
        BCB->Update();
        BCBCount->Caption = "-";
        BCBCount->Update();
        BCBProgress->Position = 0;
        BCBProgress->Update();

        for (int lib=0;lib<NUM_LIBS;lib++)
            for (int bcb=0;bcb<NUM_BCB;bcb++)
                {
                Application->HandleMessage();
                if (!deleting) break;

                BCB->Caption = "[ "+BCB_Names[bcb]+" ]";
                BCB->Update();
                BCBCount->Caption = IntToStr(bcb+1) + " of " + IntToStr(NUM_BCB);
                BCBCount->Update();
                BCBProgress->Position = bcb+1;
                BCBProgress->Update();

                // Restore DirectX Include files
                if (!deleting) break;
                if (!copied_includes[bcb] && BCB_Exists[bcb])
                    {
                    RESULT( "------------------------------------------------------------------------------------------------" );
                    ACTION( clGreen, "> Restoring DirectX 7.0a Include files to "+BCB_Names[bcb]+"\\Include" );
                    RESULT( Action->Caption );
                    RESULT( "------------------------------------------------------------------------------------------------" );

                    copied_includes[bcb] = true;
                    RestoreDirectXIncludeFiles( lib, bcb );
                    }

                // Copy DirectX Lib files
                if (!deleting) break;
                if (!copied_libs[bcb] && BCB_Exists[bcb])
                    {
                    RESULT( "------------------------------------------------------------------------------------------------" );
                    ACTION( clGreen, "> Restoring DirectX 7.0a Lib files to "+BCB_Names[bcb]+"\\Lib" );
                    RESULT( Action->Caption );
                    RESULT( "------------------------------------------------------------------------------------------------" );

                    copied_libs[bcb] = true;
                    RestoreDirectXLibFiles( lib, bcb );
                    }
                }
        }

    // if no libraries are installed, cleanup extra files
    bool something_installed = false;
    for (int lib=0;lib<NUM_LIBS;lib++)
        for (int bcb=0;bcb<NUM_BCB;bcb++)
            something_installed |= Library_Installed[lib][bcb];

    if (!something_installed)
        {
        DeleteFile( BCB_Root_Dirs[BCB1]+"\\Include\\TDx_Library_*.*" );
        DeleteFile( BCB_Root_Dirs[BCB1]+"\\Lib\\TDx_Library_*.*" );
        DeleteFile( BCB_Root_Dirs[BCB3]+"\\Include\\TDx_Library_*.*" );
        DeleteFile( BCB_Root_Dirs[BCB3]+"\\Lib\\TDx_Library_*.*" );
        DeleteFile( BCB_Root_Dirs[BCB4]+"\\Include\\TDx_Library_*.*" );
        DeleteFile( BCB_Root_Dirs[BCB4]+"\\Lib\\TDx_Library_*.*" );
        DeleteFile( BCB_Root_Dirs[BCB5]+"\\Include\\TDx_Library_*.*" );
        DeleteFile( BCB_Root_Dirs[BCB5]+"\\Lib\\TDx_Library_*.*" );
        }

    // remove the 'TDx_Library' key if required
    registry->OpenKey( TDx_Root_Key, false );
    registry->GetKeyInfo( info );
    if (info.NumSubKeys==0) registry->DeleteKey( TDx_Root_Key );
    registry->CloseKey();

    // remove the 'BCB-Tools.com' key if required
    registry->OpenKey( BCB_Root_Key, false );
    registry->GetKeyInfo( info );
    if (info.NumSubKeys==0) registry->DeleteKey( BCB_Root_Key );
    registry->CloseKey();

    // finished
    delete registry;

    Lib->Caption = "-";
    Lib->Update();
    LibCount->Caption = "-";
    LibCount->Update();
    LibProgress->Position = 0;
    LibProgress->Update();

    BCB->Caption = "-";
    BCB->Update();
    BCBCount->Caption = "-";
    BCBCount->Update();
    BCBProgress->Position = 0;
    BCBProgress->Update();

    File->Caption = "-";
    File->Update();
    FilePath->Caption = "-";
    FilePath->Update();
    FileCount->Caption = "-";
    FileCount->Update();
    FileProgress->Position = 0;
    FileProgress->Update();

    if (deleting)   // user has not clicked abort
        {
        bool any_removed_from_bcb1 = false;
        AnsiString libs = "";
        for (int lib=0;lib<NUM_LIBS;lib++)
            {
            any_removed_from_bcb1 |= Library_Installed[lib][BCB1];
            if (Library_Installed[lib][BCB1])
                libs = libs + "        \""+BCB_Root_Dirs[BCB1]+"\\Lib\\"+Library_Names[lib]+"_Install.CPP\"->\n        Remove-> \n";
            }
        if (any_removed_from_bcb1)
            {
            MainForm->InstallWav->Stop();
            MainForm->BCBCheck = false;
            Error("\
This application cannot fully automate the uninstall for BCB1.\n\
For a successful uninstall, complete the following steps:-\n\
\n\
  (1) Open BCB1\n\
\n\
  (2) Close All Files\n\
      [ File->Close All ]\n\
\n\
  (3) Remove the library\n\
      [ Component->Install-> \n"+
libs+"\
        Ok ]\n\
\n\
  (4) Close BCB1\n\
\n\
When you have finished the steps shown above, click OK.");
            ErrorOccurred = false;
            MainForm->BCBCheck = false;
            MainForm->InstallWav->Play(0,DSBPLAY_LOOPING);
            }

        RESULT( "------------------------------------------------------------------------------------------------" );
        ACTION( (TColor) 0x00D03000, "> Remove Complete" );
        RESULT( "------------------------------------------------------------------------------------------------" );
        MainForm->Completed->Play(0,0);

        Installing->Caption = "-";
        Installing->Hint = "Ready to remove";
        Installing->Update();

        SpeedButton1->Visible = true;
        CopiedLabel->Visible = true;

        // just click the install button again, with a little fix there
        RemoveComplete = true;
//        InstallClick(this);
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TUnInstallForm::RestoreDirectXIncludeFiles( int lib, int bcb )
{
FileListBox1->Mask = "*.*";
try {
    FileListBox1->Directory = BCB_Root_Dirs[bcb] + "\\Include\\TDX_SAVE";
    Installing->Caption = "DirectX Include Files";
    Installing->Hint = "Currently Restoring DirectX 7.0a SDK Files";
    Installing->Update();
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find DirectX directory "+BCB_Root_Dirs[bcb] + "\\Include\\TDX_SAVE" );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!deleting) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];
    AnsiString dest = BCB_Root_Dirs[bcb] + "\\Include\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = srce;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    RESULT( "> Restoring "+dest );
    FileCopy( srce, dest, true );
    AnsiString backup_file = BCB_Root_Dirs[bcb] + "\\Include\\TDX_SAVE\\"+FileListBox1->Items->Strings[file];
    if (FileExists( backup_file )) DeleteFile( backup_file.c_str() );
    }
RESULT( "> Remove Directory " + FileListBox1->Directory );
RemoveDirectory( FileListBox1->Directory.c_str() );
}
//---------------------------------------------------------------------------
void __fastcall TUnInstallForm::RestoreDirectXLibFiles( int lib, int bcb )
{
FileListBox1->Mask = "*.*";
try {
    FileListBox1->Directory = BCB_Root_Dirs[bcb] + "\\Lib\\TDX_SAVE";
    Installing->Caption = "DirectX Lib Files";
    Installing->Hint = "Currently Removing DirectX 7.0a SDK Lib Files";
    Installing->Update();
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find DirectX directory "+ BCB_Root_Dirs[bcb] + "\\Lib\\TDX_SAVE" );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!deleting) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];
    AnsiString dest = BCB_Root_Dirs[bcb] + "\\Lib\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = srce;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (FileExists( dest )) DeleteFile( dest );
    RESULT( "> Restoring "+dest );
    FileCopy( srce, dest, true );
    AnsiString backup_file = BCB_Root_Dirs[bcb] + "\\Lib\\TDX_SAVE\\"+FileListBox1->Items->Strings[file];
    if (FileExists( backup_file )) DeleteFile( backup_file.c_str() );
    }
RESULT( "> Remove Directory " + FileListBox1->Directory );
RemoveDirectory( FileListBox1->Directory.c_str() );
}
//---------------------------------------------------------------------------
void __fastcall TUnInstallForm::DeleteIncludeFiles( int lib, int bcb )
{
switch (lib)
    {
    case DRAW:      FileListBox1->Mask = "TDD*.H;TDx_Draw*.H"; break;
    case SOUND:     FileListBox1->Mask = "TDS*.H;TDx_Sound*.H"; break;
    case INPUT:     FileListBox1->Mask = "TDI*.H;TDx_Input*.H"; break;
    case PLAY:      FileListBox1->Mask = "TDP*.H;TDx_Play*.H"; break;
    case D3I:       FileListBox1->Mask = "TD3DI*.H;TDx_3DI*.H"; break;
    }

// BCB directories
try {
    FileListBox1->Directory = BCB_Root_Dirs[bcb] + "\\Include";
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find "+Library_Names[lib]+" v"+Library_Versions[lib]+" directory "+BCB_Root_Dirs[bcb]+"\\Include" );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!deleting) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = srce;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (FileExists( srce ))
        {
        RESULT( "> Delete "+srce );
        DeleteFile( srce.c_str() );
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TUnInstallForm::DeleteSourceCodeFiles( int lib, int bcb )
{
switch (lib)
    {
    case DRAW:      FileListBox1->Mask = "TDD*.CPP;TDx_Draw*.CPP"; break;
    case SOUND:     FileListBox1->Mask = "TDS*.CPP;TDx_Sound*.CPP"; break;
    case INPUT:     FileListBox1->Mask = "TDI*.CPP;TDx_Input*.CPP"; break;
    case PLAY:      FileListBox1->Mask = "TDP*.CPP;TDx_Play*.CPP"; break;
    case D3I:       FileListBox1->Mask = "TD3D*.CPP;TDx_3D*.CPP"; break;
    }

// BCB directories
try {
    FileListBox1->Directory = BCB_Root_Dirs[bcb] + "\\Include";
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find "+Library_Names[lib]+" v"+Library_Versions[lib]+" directory "+BCB_Root_Dirs[bcb]+"\\Include" );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!deleting) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = srce;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (FileExists( srce ))
        {
        RESULT( "> Delete "+srce );
        DeleteFile( srce.c_str() );
        }
    }


// TDx_Library directories
try {
    FileListBox1->Directory = TDx_Root_Dir + "\\Pre-Compiled\\"+BCB_Names[bcb];
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find "+Library_Names[lib]+" v"+Library_Versions[lib]+" directory "+TDx_Root_Dir+"\\Pre-Compiled\\"+BCB_Names[bcb] );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!deleting) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = srce;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (FileExists( srce ))
        {
        RESULT( "> Delete "+srce );
        DeleteFile( srce.c_str() );
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TUnInstallForm::DeleteLibFiles( int lib, int bcb )
{
switch (lib)
    {
    case DRAW:      FileListBox1->Mask = "TDD*.OBJ;TDx_Draw*.LIB;TDx_Draw*.OBJ;TDx_Draw*.RES;TDx_Draw*.CPP"; break;
    case SOUND:     FileListBox1->Mask = "TDS*.OBJ;TDx_Sound*.LIB;TDx_Sound*.OBJ;TDx_Sound*.RES;TDx_Sound*.CPP"; break;
    case INPUT:     FileListBox1->Mask = "TDI*.OBJ;TDx_Input*.LIB;TDx_Input*.OBJ;TDx_Input*.RES;TDx_Input*.CPP"; break;
    case PLAY:      FileListBox1->Mask = "TDP*.OBJ;TDx_Play*.LIB;TDx_Play*.OBJ;TDx_Play*.RES;TDx_Play*.CPP"; break;
    case D3I:       FileListBox1->Mask = "TD3D*.OBJ;TDx_3D*.LIB;TDx_3D*.OBJ;TDx_3D*.RES;TDx_3D*.CPP"; break;
    }

// BCB directories
try {
    FileListBox1->Directory = BCB_Root_Dirs[bcb] + "\\Lib";
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find "+Library_Names[lib]+" v"+Library_Versions[lib]+" directory "+BCB_Root_Dirs[bcb]+"\\Lib" );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!deleting) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = srce;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (FileExists( srce ))
        {
        RESULT( "> Delete "+srce );
        DeleteFile( srce.c_str() );
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TUnInstallForm::DeleteProjectFiles( int lib, int bcb )
{
// project files:
//  all     library.bpl.bpi.bpk.lib
//          library_install.cpp.h
//
//  BCB1\lib\library.mak
//                  .bpk
//  BCB1\lib\library.cpp
//  BCB1\lib\library.h
//  BCB1\lib\library.res
//  BCB1\lib\library_install.cpp
//  BCB1\lib\library_install.h
//                  .bpl
//                  .bpi
//  BCB1\lib\library.lib
//
//                  .mak
//  BCB3\lib\library.bpk
//  BCB3\lib\library.cpp
//  BCB3\lib\library.h
//  BCB3\lib\library.res
//  BCB3\lib\library_install.cpp
//  BCB3\lib\library_install.h
//  BCB3\lib\library.bpl
//  BCB3\lib\library.bpi
//  BCB3\lib\library.lib
//
//                  .mak
//  BCB4\lib\library.bpk
//  BCB4\lib\library.cpp
//  BCB4\lib\library.h
//  BCB4\lib\library.res
//  BCB4\lib\library_install.cpp
//  BCB4\lib\library_install.h
//  BCB4\projects\bpl\library.bpl
//  BCB4\projects\lib\library.bpi
//  BCB4\projects\lib\library.lib
//
//                  .mak                    // start first section
//  BCB5\lib\library.bpk
//  BCB5\lib\library.cpp
//  BCB5\lib\library.h
//  BCB5\lib\library.res
//  BCB5\lib\library_install.cpp
//  BCB5\lib\library_install.h              // end first section
//  BCB5\projects\bpl\library.bpl           // second section
//  BCB5\projects\lib\library.bpi           // start third section
//  BCB5\projects\lib\library.lib           // end third section
//
//                  .mak                    // start first section
//  BCB6\lib\library.bpk
//  BCB6\lib\library.cpp
//  BCB6\lib\library.h
//  BCB6\lib\library.res
//  BCB6\lib\library_install.cpp
//  BCB6\lib\library_install.h              // end first section
//  BCB6\projects\bpl\library.bpl           // second section
//  BCB6\projects\lib\library.bpi           // start third section
//  BCB6\projects\lib\library.lib           // end third section

// --- first section
switch (bcb)
    {
    case BCB1:      FileListBox1->Mask = "TDx.MAK;TDx.CPP;TDx.H;TDx.RES;TDx_Install.CPP;TDx_Install.H"; break;
    case BCB3:      FileListBox1->Mask = "TDx.BPK;TDx.CPP;TDx.H;TDx.RES;TDx_Install.CPP;TDx_Install.H"; break;
    case BCB4:      FileListBox1->Mask = "TDx.BPK;TDx.CPP;TDx.H;TDx.RES;TDx_Install.CPP;TDx_Install.H"; break;
    case BCB5:      FileListBox1->Mask = "TDx.BPK;TDx.CPP;TDx.H;TDx.RES;TDx_Install.CPP;TDx_Install.H"; break;
    case BCB6:      FileListBox1->Mask = "TDx.BPK;TDx.CPP;TDx.H;TDx.RES;TDx_Install.CPP;TDx_Install.H"; break;
    }
FileListBox1->Mask = ReplaceText( FileListBox1->Mask, "TDx", Library_Names[lib] );

// bcb
try {
    FileListBox1->Directory = BCB_Root_Dirs[bcb] + "\\Lib\\";
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find "+Library_Names[lib]+" v"+Library_Versions[lib]+" directory "+BCB_Root_Dirs[bcb] + "\\Lib\\" );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!deleting) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = srce;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (FileExists( srce ))
        {
        RESULT( "> Delete "+srce );
        DeleteFile( srce.c_str() );
        }
    }

// --- second section
switch (bcb)
    {
    case BCB1:      FileListBox1->Mask = ""; break;
    case BCB3:      FileListBox1->Mask = "TDx.BPL"; break;
    case BCB4:      FileListBox1->Mask = "TDx.BPL"; break;
    case BCB5:      FileListBox1->Mask = "TDx.BPL"; break;
    case BCB6:      FileListBox1->Mask = "TDx.BPL"; break;
    }
FileListBox1->Mask = ReplaceText( FileListBox1->Mask, "TDx", Library_Names[lib] );
// bcb
AnsiString dir;
try {
    switch (bcb)
        {
        case BCB1:  dir = BCB_Root_Dirs[bcb] + "\\Lib"; break;
        case BCB3:  dir = BCB_Root_Dirs[bcb] + "\\Lib"; break;
        case BCB4:  dir = BCB_Root_Dirs[bcb] + "\\Projects\\Bpl"; break;
        case BCB5:  dir = BCB_Root_Dirs[bcb] + "\\Projects\\Lib"; break;
        case BCB6:  dir = BCB_Root_Dirs[bcb] + "\\Projects\\Lib"; break;
        }
    FileListBox1->Directory = dir;
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find "+Library_Names[lib]+" v"+Library_Versions[lib]+" directory "+dir );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!deleting) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = srce;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (FileExists( srce ))
        {
        RESULT( "> Delete "+srce );
        DeleteFile( srce.c_str() );
        }
    }



// --- third section
switch (bcb)
    {
    case BCB1:      FileListBox1->Mask = "TDx.LIB"; break;
    case BCB3:      FileListBox1->Mask = "TDx.BPI;TDx.LIB"; break;
    case BCB4:      FileListBox1->Mask = "TDx.BPI;TDx.LIB"; break;
    case BCB5:      FileListBox1->Mask = "TDx.BPI;TDx.LIB"; break;
    case BCB6:      FileListBox1->Mask = "TDx.BPI;TDx.LIB"; break;
    }
FileListBox1->Mask = ReplaceText( FileListBox1->Mask, "TDx", Library_Names[lib] );
// bcb
dir = "";
try {
    switch (bcb)
        {
        case BCB1:  dir = BCB_Root_Dirs[bcb] + "\\Lib"; break;
        case BCB3:  dir = BCB_Root_Dirs[bcb] + "\\Lib"; break;
        case BCB4:  dir = BCB_Root_Dirs[bcb] + "\\Projects\\Bpl"; break;
        case BCB5:  dir = BCB_Root_Dirs[bcb] + "\\Projects\\Lib"; break;
        case BCB6:  dir = BCB_Root_Dirs[bcb] + "\\Projects\\Lib"; break;
        }
    FileListBox1->Directory = dir;
    }
catch (...)
    {
    RESULT( "> ERROR: Could not find "+Library_Names[lib]+" v"+Library_Versions[lib]+" directory "+dir );
    return;
    }

FileProgress->Min = 0;
FileProgress->Max = FileListBox1->Items->Count;
FileProgress->Position = 0;
FileProgress->Update();

for (int file=0;file<FileListBox1->Items->Count;file++)
    {
    Application->HandleMessage();
    if (!deleting) break;

    AnsiString srce = FileListBox1->Directory + "\\" + FileListBox1->Items->Strings[file];

    File->Caption = "[ "+FileListBox1->Items->Strings[file]+" ]";
    File->Update();
    FilePath->Caption = srce;
    FilePath->Update();
    FileCount->Caption = IntToStr(file+1) + " of " + IntToStr(FileListBox1->Items->Count);
    FileCount->Update();
    FileProgress->Position = file;
    FileProgress->Update();

    if (FileExists( srce ))
        {
        RESULT( "> Delete "+srce );
        DeleteFile( srce.c_str() );
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TUnInstallForm::UnRegisterComponents( int lib, int bcb )
{
TRegistry* registry = new TRegistry();
if (registry==NULL)
    {
    Error( "\
----------------------------------------\n\
Remove Components Problem:\n\
----------------------------------------\n\
\n\
The components cannot be removed because there is not enough memory available to access the system registry.\n\
Try reducing the number of applications you currently have open, or buy more memory.\n\
\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should install successfully.\n\
\n\
Error Code: RCOM_1\n\
\n\
----------------------------------------\n\
" );
    return;
    }
registry->RootKey = HKEY_LOCAL_MACHINE;
registry->LazyWrite = false;
TRegKeyInfo info;

switch (bcb)
    {
    case BCB1:

        ErrorOccurred = false;
        UnInstallBCB1( lib );
        Library_Installed[lib][BCB1] = false;
        Library_Installed_Version[lib][BCB1] = "0.00";
        if (ErrorOccurred)
            {
            RESULT( "> ERROR:  BCB1 Remove Components Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->DeleteValue( "BCB1" );
            registry->DeleteValue( "BCB1_Version" );
            registry->DeleteValue( "BCB1_First_Installed" );
            registry->GetKeyInfo( info );
            if (info.NumValues==0) registry->DeleteKey( TDx_Root_Key+"\\"+Library_Names[lib] );
            registry->CloseKey();
            RESULT( "> BCB1 Components Removed Successfully!" );
            }
        break;

    case BCB3:

        ErrorOccurred = false;
        UnInstallBCB3( lib );
        Library_Installed[lib][BCB3] = false;
        Library_Installed_Version[lib][BCB3] = "0.00";
        if (ErrorOccurred)
            {
            RESULT( "> ERROR:  BCB3 Remove Components Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->DeleteValue( "BCB3" );
            registry->DeleteValue( "BCB3_Version" );
            registry->DeleteValue( "BCB3_First_Installed" );
            registry->GetKeyInfo( info );
            if (info.NumValues==0) registry->DeleteKey( TDx_Root_Key+"\\"+Library_Names[lib] );
            registry->CloseKey();
            RESULT( "> BCB3 Components Removed Successfully!" );
            }
        break;

    case BCB4:

        ErrorOccurred = false;
        UnInstallBCB4( lib );
        Library_Installed[lib][BCB4] = false;
        Library_Installed_Version[lib][BCB4] = "0.00";
        if (ErrorOccurred)
            {
            RESULT( "> ERROR:  BCB4 Remove Components Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->DeleteValue( "BCB4" );
            registry->DeleteValue( "BCB4_Version" );
            registry->DeleteValue( "BCB4_First_Installed" );
            registry->GetKeyInfo( info );
            if (info.NumValues==0) registry->DeleteKey( TDx_Root_Key+"\\"+Library_Names[lib] );
            registry->CloseKey();
            RESULT( "> BCB4 Components Removed Successfully!" );
            }
        break;

    case BCB5:

        ErrorOccurred = false;
        UnInstallBCB5( lib );
        Library_Installed[lib][BCB5] = false;
        Library_Installed_Version[lib][BCB5] = "0.00";
        if (ErrorOccurred)
            {
            RESULT( "> ERROR:  BCB5 Remove Components Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->DeleteValue( "BCB5" );
            registry->DeleteValue( "BCB5_Version" );
            registry->DeleteValue( "BCB5_First_Installed" );
            registry->GetKeyInfo( info );
            if (info.NumValues==0) registry->DeleteKey( TDx_Root_Key+"\\"+Library_Names[lib] );
            registry->CloseKey();
            RESULT( "> BCB5 Components Removed Successfully!" );
            }
        break;

    case BCB6:

        ErrorOccurred = false;
        UnInstallBCB6( lib );
        Library_Installed[lib][BCB5] = false;
        Library_Installed_Version[lib][BCB5] = "0.00";
        if (ErrorOccurred)
            {
            RESULT( "> ERROR:  BCB6 Remove Components Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->DeleteValue( "BCB6" );
            registry->DeleteValue( "BCB6_Version" );
            registry->DeleteValue( "BCB6_First_Installed" );
            registry->GetKeyInfo( info );
            if (info.NumValues==0) registry->DeleteKey( TDx_Root_Key+"\\"+Library_Names[lib] );
            registry->CloseKey();
            RESULT( "> BCB6 Components Removed Successfully!" );
            }
        break;

    }

delete registry;
Application->HandleMessage();
}
//---------------------------------------------------------------------------
void __fastcall TUnInstallForm::UnRegisterComponentHelp( int lib, int bcb )
{
TRegistry* registry = new TRegistry();
if (registry==NULL)
    {
    Error( "\
--------------------------------------------\n\
Remove Component Help Problem:\n\
--------------------------------------------\n\
\n\
The help files cannot be removed because there is not enough memory available to access the system registry.\n\
Try reducing the number of applications you currently have open, or buy more memory.\n\
\n\
Once the problem is resolved the "+Library_Names[lib]+" v"+Library_Versions[lib]+" should install successfully.\n\
\n\
Error Code: RHELP_1\n\
\n\
--------------------------------------------\n\
" );
    return;
    }
registry->RootKey = HKEY_LOCAL_MACHINE;
registry->LazyWrite = false;
TRegKeyInfo info;

switch (bcb)
    {
    case BCB1:

        ErrorOccurred = false;
        UnInstallBCB1Help( lib );
        Library_Installed_Help[lib][BCB1] = false;
        if (ErrorOccurred)
            {
            RESULT( "> ERROR:  BCB1 Remove Help Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->DeleteValue( "BCB1_Help" );
            registry->GetKeyInfo( info );
            if (info.NumValues==0) registry->DeleteKey( TDx_Root_Key+"\\"+Library_Names[lib] );
            registry->CloseKey();
            RESULT( "> BCB1 Help Removed Successfully!" );
            }
        break;

    case BCB3:

        ErrorOccurred = false;
        UnInstallBCB3Help( lib );
        Library_Installed_Help[lib][BCB3] = false;
        if (ErrorOccurred)
            {
            RESULT( "> ERROR:  BCB3 Remove Help Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->DeleteValue( "BCB3_Help" );
            registry->GetKeyInfo( info );
            if (info.NumValues==0) registry->DeleteKey( TDx_Root_Key+"\\"+Library_Names[lib] );
            registry->CloseKey();
            RESULT( "> BCB3 Help Removed Successfully!" );
            }
        break;

    case BCB4:

        ErrorOccurred = false;
        UnInstallBCB4Help( lib );
        Library_Installed_Help[lib][BCB4] = false;
        if (ErrorOccurred)
            {
            RESULT( "> ERROR:  BCB4 Remove Help Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->DeleteValue( "BCB4_Help" );
            registry->GetKeyInfo( info );
            if (info.NumValues==0) registry->DeleteKey( TDx_Root_Key+"\\"+Library_Names[lib] );
            registry->CloseKey();
            RESULT( "> BCB4 Help Removed Successfully!" );
            }
        break;

    case BCB5:

        ErrorOccurred = false;
        UnInstallBCB5Help( lib );
        Library_Installed_Help[lib][BCB5] = false;
        if (ErrorOccurred)
            {
            RESULT( "> ERROR:  BCB5 Remove Help Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->DeleteValue( "BCB5_Help" );
            registry->GetKeyInfo( info );
            if (info.NumValues==0) registry->DeleteKey( TDx_Root_Key+"\\"+Library_Names[lib] );
            registry->CloseKey();
            RESULT( "> BCB5 Help Removed Successfully!" );
            }
        break;

    case BCB6:

        ErrorOccurred = false;
        UnInstallBCB6Help( lib );
        Library_Installed_Help[lib][BCB5] = false;
        if (ErrorOccurred)
            {
            RESULT( "> ERROR:  BCB6 Remove Help Failed!" );
            }
        else
            {
            registry->OpenKey( TDx_Root_Key+"\\"+Library_Names[lib], true );
            registry->DeleteValue( "BCB6_Help" );
            registry->GetKeyInfo( info );
            if (info.NumValues==0) registry->DeleteKey( TDx_Root_Key+"\\"+Library_Names[lib] );
            registry->CloseKey();
            RESULT( "> BCB6 Help Removed Successfully!" );
            }
        break;

    }

delete registry;
Application->HandleMessage();
}
//---------------------------------------------------------------------------
#include "UnInstallBCB1.inc"
#include "UnInstallBCB3.inc"
#include "UnInstallBCB4.inc"
#include "UnInstallBCB5.inc"
#include "UnInstallBCB6.inc"
#include "UnInstallBCB1Help.inc"
#include "UnInstallBCB3Help.inc"
#include "UnInstallBCB4Help.inc"
#include "UnInstallBCB5Help.inc"
#include "UnInstallBCB6Help.inc"
//---------------------------------------------------------------------------

void __fastcall TUnInstallForm::SpeedButton1Click(TObject *Sender)
{
TClipboard* clip = Clipboard();
clip->AsText = Results->Items->Text;
CopiedLabel->Caption = "( copied )";
CopiedLabel->Update();
Application->HandleMessage();
}
//---------------------------------------------------------------------------
void __fastcall TUnInstallForm::BCBToolsClick(TObject *Sender)
{
MainForm->BCBTools->OnClick(this);
}
//---------------------------------------------------------------------------


void __fastcall TUnInstallForm::FormPaint(TObject *Sender)
{
Canvas->Brush->Color = clBlack;
Canvas->Brush->Style = bsSolid;
Canvas->FrameRect( ClientRect );
}
//---------------------------------------------------------------------------
void __fastcall TUnInstallForm::UninstallClick(TObject *Sender)
{
MainForm->Prev->Play(0,0);
}
//---------------------------------------------------------------------------
void __fastcall TUnInstallForm::CancelClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TUnInstallForm::InstallMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
if (!Active) return;
TComponent* sender = (TComponent*) Sender;
if (sender==NULL) return;

// if there is an image that is already Highlighted, un-highlight it now.
if ((sender->Name!="Install") && (Highlighted=="Install")) { InstallImage->Picture = MainForm->Lo->Picture; Install->Font->Color = clNavy; }
else if ((sender->Name!="Prev") && (Highlighted=="Prev")) { PrevImage->Picture = MainForm->Lo->Picture; Prev->Font->Color = clNavy; }
else if ((sender->Name!="Exit") && (Highlighted=="Exit")) { ExitImage->Picture = MainForm->Lo->Picture; Exit->Font->Color = clNavy; }

// if the image is not already Highlighted, but now needs to be Highlighted....
if ((sender->Name=="Install") && (Highlighted != "Install")) { InstallImage->Picture = MainForm->Hi->Picture; Install->Font->Color = clWhite; if (MainForm->Pop->Playing) MainForm->Pop->Stop(); MainForm->Pop->Play(0,0); }
else if ((sender->Name=="Prev") && (Highlighted!="Prev")) { PrevImage->Picture = MainForm->Hi->Picture; Prev->Font->Color = clWhite; if (MainForm->Pop->Playing) MainForm->Pop->Stop(); MainForm->Pop->Play(0,0); }
else if ((sender->Name=="Exit") && (Highlighted!="Exit")) { ExitImage->Picture = MainForm->Hi->Picture; Exit->Font->Color = clWhite; if (MainForm->Pop->Playing) MainForm->Pop->Stop(); MainForm->Pop->Play(0,0); }

// remember which button is Highlighted
Highlighted = sender->Name;
}
//---------------------------------------------------------------------------
void __fastcall TUnInstallForm::PrevClick(TObject *Sender)
{
if (!deleting)
    {
    if (MainForm->Prev->Playing) MainForm->Prev->Stop();
    MainForm->Prev->Play(0,0);
    MainForm->GetCurrentInstallStatus();
    Close();
    }
}
//---------------------------------------------------------------------------
void __fastcall TUnInstallForm::FormShow(TObject *Sender)
{
Top = MainForm->Top;
Left = MainForm->Left;
Width = MainForm->Width;
Height = MainForm->Height;
}
//---------------------------------------------------------------------------

void __fastcall TUnInstallForm::TitleClick(TObject *Sender)
{
MainForm->TitleClick(this);
}
//---------------------------------------------------------------------------

